/****************************************************************************/

#include "nfi.h"

/****************************************************************************/

// This is the message dispatching class for the extract dialog box
class CExtractDialogDispatcher : public CDispatcher {
public:
    CExtractDialogDispatcher (handlerType Handler) : CDispatcher (Handler) {}

    LRESULT MsgInit (HWND, UINT, WPARAM, LPARAM);
    LRESULT MsgHelp (HWND, UINT, WPARAM, LPARAM);

    LRESULT CmdBrowse (HWND, WORD, WORD, HWND);
    LRESULT CmdDone   (HWND, WORD, WORD, HWND);

MESSAGE_LIST;
COMMAND_LIST;
};

START_MESSAGE_LIST (CExtractDialogDispatcher)
    MESSAGE (WM_HELP       , MsgHelp)
    MESSAGE (WM_INITDIALOG , MsgInit)
END_MESSAGE_LIST

START_COMMAND_LIST (CExtractDialogDispatcher)
    COMMAND (IDC_BROWSE , CmdBrowse)
    COMMAND (IDOK       , CmdDone)
    COMMAND (IDCANCEL   , CmdDone)
END_COMMAND_LIST

/****************************************************************************/

static CExtractDialogDispatcher DialogDispatch (handlerNone);
static char SelectedExtractDir[MAX_PATH]="";

/*****************************************************************************
 * Name         - ExtractProc
 * Purpose      - This handles messages for the extract dialog box by using
 *                it's dispatcher class.
 * Arguments    - Standard window procedure dialog boxes
 * Return Value - Depends on the message
 ****************************************************************************/
LRESULT CALLBACK ExtractProc (HWND hDlg , UINT uMessage ,
                              WPARAM wParam, LPARAM lParam)
{
 return DialogDispatch.dispatchMessage (hDlg , uMessage , wParam , lParam);
}

/*****************************************************************************
 * Name         - CExtractDialogDispatcher::MsgInit
 * Purpose      - This handles the WM_INITDIALOG message.
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT CExtractDialogDispatcher::MsgInit (HWND hDlg , UINT /*uMessage*/ ,
                                           WPARAM /*wParam*/ , LPARAM /*lParam*/)
{
 // Tell the buttons to use the proper bitmap
 SendMessage (GetDlgItem (hDlg , IDOK) , BM_SETIMAGE , (WPARAM) IMAGE_BITMAP , (LPARAM) hOKBitmap);
 SendMessage (GetDlgItem (hDlg , IDCANCEL) , BM_SETIMAGE , (WPARAM) IMAGE_BITMAP , (LPARAM) hCANCELBitmap);
 SendMessage (GetDlgItem (hDlg , IDC_BROWSE) , BM_SETIMAGE , (WPARAM) IMAGE_BITMAP , (LPARAM) hBROWSEBitmap);

 // Tell the edit text box that holds the extraction directory the maximum
 // text it is allowed to hold, then put the default extraction text in it.
 // Note we use MAX_PATH -1 because MAX_PATH takes the null into account
 SetDlgItemText (hDlg , IDE_EXTRACTIONDIR , Setup.ExtractDir);
 SendMessage (GetDlgItem (hDlg , IDE_EXTRACTIONDIR) , EM_SETLIMITTEXT ,
              MAX_PATH - 1 , 0);

 // If there is a selected file, we check the "Extract selection" radio button.
 // Otherwise, check "Extract All" and disable the two items that have something
 // to do with using the selection, since there isn't one
 if (ListView_GetSelectedCount (hwndListview) > 0)
     CheckDlgButton (hDlg , IDC_EXTRACTSELECTED , BST_CHECKED);
 else {
     CheckDlgButton (hDlg , IDC_EXTRACTALL , BST_CHECKED);
     EnableWindow (GetDlgItem (hDlg , IDC_EXTRACTSELECTED) , FALSE);
     EnableWindow (GetDlgItem (hDlg , IDC_EXTRACTBUTSEL) , FALSE);
 }

 // Check the proper "Space Extraction" parameter based on the global setup
 // options.
 CheckDlgButton (hDlg , IDC_EXTRACTKEEP + Setup.ExtractSpaceProcess , BST_CHECKED);

 // Check the "Maintain directory structure" button, if needed
 CheckDlgButton (hDlg , IDC_MAINTAINDIR , Setup.MaintainDirStructure ? BST_CHECKED : BST_UNCHECKED);

 // Center the dialog over the application window
 CenterWindow (hDlg , GetWindow (hDlg , GW_OWNER));

 return TRUE;
}

/*****************************************************************************
 * Name         - CExtractDialogDispatcher::MsgHelp
 * Purpose      - This handles the WM_HELP message.
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT CExtractDialogDispatcher::MsgHelp (HWND hDlg , UINT /*uMessage*/ ,
                                           WPARAM /*wParam*/ , LPARAM lParam)
{
 static DWORD HelpID[] = {
     IDE_EXTRACTIONDIR   , IDE_EXTRACTIONDIR   ,
     IDC_BROWSE          , IDC_BROWSE          ,
     IDC_EXTRACTALL      , IDC_EXTRACTALL      ,
     IDC_EXTRACTSELECTED , IDC_EXTRACTSELECTED ,
     IDC_EXTRACTBUTSEL   , IDC_EXTRACTBUTSEL   ,
     IDC_EXTRACTKEEP     , IDC_EXTRACTKEEP     ,
     IDC_EXTRACTCONVERT  , IDC_EXTRACTCONVERT  ,
     IDC_EXTRACTREMOVE   , IDC_EXTRACTREMOVE   ,
     IDC_MAINTAINDIR     , IDC_MAINTAINDIR     ,
     IDC_EXTRACTOK       , IDC_EXTRACTOK       ,
     IDC_EXTRACTCANCEL   , IDC_EXTRACTCANCEL   ,
     IDOK                , IDC_ADDOK           ,
     IDCANCEL            , IDC_ADDCANCEL       ,
     0                   , 0                   ,
 };

 char HelpFile[MAX_PATH];
 GetHelpFileName (HelpFile);
 HELPINFO hi = *(LPHELPINFO) lParam;

 if (hi.iContextType == HELPINFO_WINDOW &&
     hDlg != hi.hItemHandle) {
         WinHelp ((HWND) hi.hItemHandle ,
                  HelpFile ,
                  HELP_WM_HELP ,
                  (DWORD) (LPVOID) HelpID);
 }

 return TRUE;
}

/*****************************************************************************
 * Name         - CExtractDialogDispatcher::CmdDone
 * Purpose      - This function handles command messages for the button and
 *                the standard windows close controls.
 * Arguments    - hDlg: Handle of the dialog box
 *                wCommand: COmmand ID of the pressed button
 * Return Value - TRUE
 ****************************************************************************/
LRESULT CExtractDialogDispatcher::CmdDone (HWND hDlg, WORD wCommand,
                                           WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 if (wCommand == IDOK) {
     GetDlgItemText (hDlg , IDE_EXTRACTIONDIR , Setup.ExtractDir , MAX_PATH);
     if (Setup.ExtractDir[0] == '\0') {
         CResString Title   (hInst , IDS_NOEXTRACTTITLE);
         CResString Message (hInst , IDS_NOEXTRACTDIR);
         MessageBox (hDlg , Message  , Title , MB_OK | MB_ICONERROR);
         return TRUE;
     }

     // Append a trailing slash if there is not one.
     char LastChar = Setup.ExtractDir[strlen (Setup.ExtractDir) - 1];
     if (LastChar != '\\' && LastChar != '/')
         strcat (Setup.ExtractDir , "\\");

     if (IsDlgButtonChecked (hDlg , IDC_EXTRACTKEEP) == BST_CHECKED)
         Setup.ExtractSpaceProcess = 0;
     else
         if (IsDlgButtonChecked (hDlg , IDC_EXTRACTCONVERT) == BST_CHECKED)
             Setup.ExtractSpaceProcess = 1;
         else
             Setup.ExtractSpaceProcess = 2;

     if (IsDlgButtonChecked (hDlg , IDC_EXTRACTALL) == BST_CHECKED)
         ExtractProcess = 0;
     else
         if (IsDlgButtonChecked (hDlg , IDC_EXTRACTSELECTED) == BST_CHECKED)
             ExtractProcess = 1;
         else
             ExtractProcess = 2;

     Setup.MaintainDirStructure = (IsDlgButtonChecked (hDlg , IDC_MAINTAINDIR) == BST_CHECKED);

     SaveSetupStructToRegistry (Setup);
 }

 // End the dialog
 EndDialog (hDlg , wCommand);
 return TRUE;
}

/*****************************************************************************
 * Name         - ShellBrowserCallback
 * Purpose      - This is a callback function used by the directory broswer
 *                dialog. It is given messages that would let us track what
 *                the dialog is doing. Instead, we use it to force the selected
 *                dir to be the same one that was selected last time, since
 *                there is no easy way to do so.
 * Arguments    - hWnd: Handle of the browse dialog box
 *                uMessage: Message it is giving us
 * Return Value - 0
 ****************************************************************************/
static int CALLBACK ShellBrowserCallback (HWND hWnd , UINT uMessage ,
                                          LPARAM /*lParam*/ , LPARAM /*lpData*/)
{
 // Is the dialog telling us that it has been initialized, AND do we have
 // a previously selected directory?
 if (uMessage == BFFM_INITIALIZED && SelectedExtractDir[0] != '\0') {
     // Yes. Tell it to select the directory which is identified by the
     // string that the last call to the directory browser gave us. This
     // eases the user's burden of adding more than one directory from the
     // same relative position. Nice for people like me who have drives out
     // the wazoo
     SendMessage (hWnd , BFFM_SETSELECTION ,
                  (WPARAM) TRUE , (LPARAM) SelectedExtractDir);
 }

 return 0;
}

/*****************************************************************************
 * Name         - CExtractDialogDispatcher::CmdAddDir
 * Purpose      - This function handles command messages for the IDC_BROWSE
 *                button
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT CExtractDialogDispatcher::CmdBrowse (HWND hDlg, WORD /*wCommand*/,
                                             WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 BROWSEINFO bi;
 CResString Help (hInst , IDS_EXTRACTDIRHELP);

 // Set up the browse info struct. This tells the shell directory browser
 // dialog box how to add
 bi.hwndOwner = hDlg;
 bi.pidlRoot = NULL;
 bi.pszDisplayName = SelectedExtractDir;
 bi.lpszTitle = Help;
 bi.ulFlags = BIF_RETURNONLYFSDIRS;
 bi.lpfn = (BFFCALLBACK) ShellBrowserCallback;

 // Let the user select a folder
 LPITEMIDLIST pidl = SHBrowseForFolder (&bi);

 // If nothing was selected, return
 if (!pidl)
     return TRUE;

 // See if we can manage to get directory out of the identifier list. Note
 // that I don't think is is actually needed, but all the examples I've
 // ever seen do it.
 BOOL bSuccess = SHGetPathFromIDList (pidl , SelectedExtractDir);

 // Try to get a handle to the allocator the shell uses. If it doesn' work,
 // toss an error.
 IMalloc* pMalloc;
 if (SHGetMalloc (&pMalloc) != NOERROR)
     resThrow (hInst , IDS_EXCEPTMALLOC);

 // Release the id list and then release the allocator
 pMalloc->Free (pidl);
 pMalloc->Release();

 // If we actually got a folder, then add it to the text control
 if (bSuccess) {
     SendMessage (GetDlgItem (hDlg , IDE_EXTRACTIONDIR) ,
                  WM_SETTEXT , 0 , (LPARAM) SelectedExtractDir);
 }

 return TRUE;
}

/****************************************************************************/

