/****************************************************************************/

#include "nfi.h"

/****************************************************************************/

// Global variable declarations
HINSTANCE   hInst=NULL;
HACCEL      hAccelTable=NULL;
char        szAppName[50]="";
char        szTitle[120]="";
HBITMAP     hToolbarBitmap=NULL;
HBITMAP     hOKBitmap=NULL;
HBITMAP     hCANCELBitmap=NULL;
HBITMAP     hADDFILEBitmap=NULL;
HBITMAP     hADDDIRBitmap=NULL;
HBITMAP     hREMOVEBitmap=NULL;
HBITMAP     hBROWSEBitmap=NULL;
HWND        hwndMain=NULL;
HWND        hwndToolbar=NULL;
HWND        hwndStatusbar=NULL;
HWND        hwndListview=NULL;
HWND        hwndProgressDlg=NULL;
HWND        hwndProgressCtrl=NULL;
HWND        hwndTextCtrl=NULL;
HWND        hwndAddListBox=NULL;
BOOL        FileIsDirty=FALSE;
char        Filename[MAX_PATH]="";
HANDLE      MainNFIFile=INVALID_HANDLE_VALUE;
LONG        FilesInNFI=0;
LONG        FilesToAdd=0;
LONG        FileSelectedCount=0;
NFIDirEntry *DirectoryRoot=NULL;
PathInfo    *NFIPathInfo=NULL;
AddedFile   *AddedFileList=NULL;
int         ExtractProcess=0;
SetupStruct Setup = {
    0   , // When adding files, 0 == keep spaces, 1 == Convert, 2 == Remove
    0   , // When extractng, 0 == keep spaces, 1 == Convert, 2 == Remove
    1   , // When adding files, FALSE == Keep case, TRUE == Make lowercase
    1   , // Save window pos on exit, FALSE == NO, TRUE == YES
    0   , // Reopen last file on start, FALSE == NO, TRUE == YES
    1   , // Recurse dirs on adding, FALSE == NO, TRUE == YES
    1   , // When extracting, TRUE == Maintain dirs, FALSE == Don't
    ""  , // Directory to extract files to
    "/" , // Directory to add files to
    ""  , // Last file loaded
};

/*****************************************************************************
 * Name         - InitApplication
 * Purpose      - This function takes all of the steps neccesary to get the
 *                application up and running. By the time it has ended, the
 *                window is on the screen and ready to go.
 * Arguments    - hInstance: Instance handle of the program
 *                nCmdShow: Initial show state for the main window
 * Return Value - TRUE on success or FALSE on failure
 ****************************************************************************/
BOOL InitApplication (HINSTANCE hInstance , char *CmdLine , int nCmdShow)
{
 CWindowClass wc (WndProc , IDS_CLASSNAME , hInstance);

 // Save the instance handle in static variable, which will be used in
 // many subsequence calls from this application to Windows.
 hInst = hInstance;

 // Create an association if needed.
 CreateAssociation ();

 // Make sure the common controls are ready to go
 InitCommonControls ();

 // If we let the toolbar control load the bitmap itself it assumes that we
 // where braindead when drawing it and makes the colors whatever it likes
 // for whatever button. By loading the bitmap ourself from the resource
 // file we can tell it to load the bitmap as transparent. This tells it to
 // look at the first pixel in the file, and convert all pixels of that color
 // to the color of a window background. By also specifying the map 3d colors
 // flag, we tell it that instead of using the window background color it
 // should use the button face color. The upshot is that our buttons will
 // not only be in the colors we drew them in, but in addition, will look
 // correct regardless of the color scheme used.
 hToolbarBitmap = (HBITMAP) LoadImage (hInst , MAKEINTRESOURCE (IDB_TOOLBAR) ,
                                       IMAGE_BITMAP , 0 , 0 ,
                                       LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT);
 hOKBitmap = (HBITMAP) LoadImage (hInst , MAKEINTRESOURCE (IDB_OKBUTTON) ,
                                  IMAGE_BITMAP , 0 , 0 ,
                                  LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT);
 hCANCELBitmap = (HBITMAP) LoadImage (hInst , MAKEINTRESOURCE (IDB_CANCELBUTTON) ,
                                      IMAGE_BITMAP , 0 , 0 ,
                                      LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT);
 hADDFILEBitmap = (HBITMAP) LoadImage (hInst , MAKEINTRESOURCE (IDB_ADDFILEBUTTON) ,
                                       IMAGE_BITMAP , 0 , 0 ,
                                       LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT);
 hADDDIRBitmap = (HBITMAP) LoadImage (hInst , MAKEINTRESOURCE (IDB_ADDDIRBUTTON) ,
                                      IMAGE_BITMAP , 0 , 0 ,
                                      LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT);
 hREMOVEBitmap = (HBITMAP) LoadImage (hInst , MAKEINTRESOURCE (IDB_REMOVEBUTTON) ,
                                      IMAGE_BITMAP , 0 , 0 ,
                                      LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT);
 hBROWSEBitmap = (HBITMAP) LoadImage (hInst , MAKEINTRESOURCE (IDB_BROWSEBUTTON) ,
                                      IMAGE_BITMAP , 0 , 0 ,
                                      LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT);

 if (!hToolbarBitmap || !hOKBitmap || !hCANCELBitmap || !hADDFILEBitmap ||
     !hADDDIRBitmap || !hREMOVEBitmap || !hBROWSEBitmap)
     resThrow (hInst , IDS_EXCEPTBITMAPLOAD);

 // Load the application name and window title strings.
 CResString ClassStr (hInst , IDS_CLASSNAME , szAppName , sizeof (szAppName));
 CResString Title    (hInst , IDS_TITLETEXT , szTitle , sizeof (szTitle));

 // The class defaults everthing based on the usual defaults and the
 // information given in the constructor. Change the things that aren't
 // set to default manually.
 wc.setStyle (CS_HREDRAW | CS_VREDRAW);
 wc.setMenu (IDM_MAINMENU);
 wc.setResIcons (IDI_MAINICON);

 // Register the window class and return FALSE if unsuccesful.
 if (!wc.registerClass ())
     return FALSE;

 // Create a main window for this application instance.
 hwndMain = CreateWindow (szAppName,           // Class name
                          "",                  // Text for window title bar
                          WS_OVERLAPPEDWINDOW, // Window style
                          CW_USEDEFAULT, 0,    // Use default positioning
                          CW_USEDEFAULT, 0,    // Use default size
                          NULL,                // Overlapped has no parent
                          NULL,                // Use the window class menu
                          hInst,               // This instance owns this window
                          NULL);               // Don't need data in WM_CREATE

 // If window could not be created, return "failure"
 if (!hwndMain)
     return FALSE;

 if (!CreateToolbar (hwndMain))
     return FALSE;

 if (!CreateStatusbar (hwndMain))
     return FALSE;

 if (!CreateListview (hwndMain))
     return FALSE;

 // Get setup information from the registry. Every setting that the program
 // saves and restores across sesstions is stored in the registry and read
 // into the global variable on startup. Whenever the options are changed,
 // the dialog box responsible for the change saves the new settings to the
 // registry. The main window's position (if the user wants it saved), is
 // done when the window is about to close itself.
 LoadSetupStructFromRegistry (Setup);

 // Put the untitled filename into the caption bar
 MergeFilename (hwndMain , szTitle , Filename);

 // If we where supposed to save the window's position, try to load it from
 // the registry and use it to set the window's position.
 WINDOWPLACEMENT wp;
 if (Setup.SaveWindowPos && LoadMainWindowPostion (wp)) {
     // The position existed and we loaded it in. Set the window placement
     // using that information. Before we do that, we make sure that it will
     // actually be visible on the screen.

     // Get the dimensions of the screen.
     int SX = GetSystemMetrics (SM_CXSCREEN);
     int SY = GetSystemMetrics (SM_CYSCREEN);

     // First, make sure the minimized position of the window is within the
     // screen bounds.
     if (wp.ptMinPosition.x > SX)
         wp.ptMinPosition.x = SX / 2;
     if (wp.ptMinPosition.y > SY)
         wp.ptMinPosition.y = SY / 2;

     // Next, make sure the maximized position of the window is within screen
     // bounds.
     if (wp.ptMaxPosition.x > SX)
         wp.ptMaxPosition.x = SX / 2;
     if (wp.ptMaxPosition.y > SY)
         wp.ptMaxPosition.y = SY / 2;

     // Finally, make sure that the normal postion of the window falls withiin
     // screen bounds.
     if (wp.rcNormalPosition.left > SX)
         wp.rcNormalPosition.left = SX / 2;
     if (wp.rcNormalPosition.right > SX)
         wp.rcNormalPosition.right = SX;
     if (wp.rcNormalPosition.top > SY)
         wp.rcNormalPosition.top = SY / 2;
     if (wp.rcNormalPosition.bottom > SY)
         wp.rcNormalPosition.bottom = SY;

     // Now, set the window's placement
     SetWindowPlacement (hwndMain , &wp);
 } else
     ShowWindow (hwndMain , nCmdShow);

 // Update the window
 UpdateWindow (hwndMain);

 // Add common paths to the path binary tree used in the add dialog.
 NFIPathInfo = AddPathToTree (NFIPathInfo , "/");
 NFIPathInfo = AddPathToTree (NFIPathInfo , Setup.AddDir);

 // Now that the window is visible, we pull a little technical jiggery-pokery.
 // If there is a command line, we assume it's a file and open it. Otherwise,
 // we do one of two things. We either reopen the last file, if needed, or
 // do global initializations to get the program state ready.
 if (CmdLine[0] != '\0') {
     // There's a command line. Try to open the file given
     strcpy (Filename , CmdLine);
     DoOpen (FALSE);
 } else
     if (Setup.ReopenLast) {
         // If there is a last filename, copy it into the Filename
         // variable, and Call DoOpen to open the file.
         if (Setup.LastFileName[0] != '\0') {
             strcpy (Filename , Setup.LastFileName);
             DoOpen (FALSE);
         }
     }

 return TRUE;
}

/****************************************************************************/

