/****************************************************************************/

#include "nfi.h"

/*****************************************************************************
 * Name         - InsertColumn
 * Purpose      - This is
 * Arguments    - Caption: The text to put onto the column header
 *                Width: The width to use for this column (Pixels)
 *                Index: The index this column should be, starting with 0
 *                       on the left. Also is used to associate a sub item
 *                       of the inserted items with the column, so that we
 *                       know what field to sort by when the user clicks on
 *                       a column.
 * Return Value - None
 ****************************************************************************/
void InsertColumn (const char *Caption , int Width , int Index)
{
 WINASSERT (Caption);

 LV_COLUMN lvc;

 int Format = LVCFMT_RIGHT;
 if (Index == 0 || Index == 3)
     Format = LVCFMT_LEFT;

 // Initialize the LV_COLUMN structure.
 lvc.mask     = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
 lvc.fmt      = Format;
 lvc.cx       = Width;
 lvc.pszText  = (char *) Caption;
 lvc.iSubItem = Index;

 ListView_InsertColumn (hwndListview , Index , &lvc);
}

/*****************************************************************************
 * Name         - CreateListview
 * Purpose      - Creates the listview
 * Arguments    - hWnd: handle to the parent window to use when making the
 *                      listview
 * Return Value - TRUE if the listview was made
 *                FALSE on error.
 ****************************************************************************/
BOOL CreateListview (HWND hWnd)
{
 WINASSERT (hWnd);
 WINASSERT (hwndListview == NULL);

 hwndListview = CreateWindowEx (WS_EX_CLIENTEDGE ,
                              WC_LISTVIEW ,
                              "" ,
                              WS_VISIBLE | WS_CHILD |
                              LVS_EDITLABELS | LVS_REPORT | LVS_SHOWSELALWAYS ,
                              0 , 0 ,
                              0 ,
                              0 ,
                              hWnd ,
                              (HMENU) ID_LISTVIEW ,
                              hInst ,
                              NULL);

 if (hwndListview) {
     // Create an image list and attach it to the listview control.
     HIMAGELIST hil;
     HICON      hic;

     // Create the image list and add the icons to it
     hil = ImageList_Create (16 , 16 , ILC_COLORDDB | ILC_MASK , 4 , 4);

     hic = LoadIcon (hInst , MAKEINTRESOURCE (IDI_FILEICON));
     if (hic == NULL)
         resThrow (hInst , IDS_EXCEPTICONIMAGELIST);
     ImageList_AddIcon (hil , hic);
     hic = LoadIcon (hInst , MAKEINTRESOURCE (IDI_ADDEDICON));
     if (hic == NULL)
         resThrow (hInst , IDS_EXCEPTICONIMAGELIST);
     ImageList_AddIcon (hil , hic);

     // Tell the Listview to use the image list we have created. It will
     // take ownership of the image list and destroy it on it's own.
     ListView_SetImageList (hwndListview , hil , LVSIL_SMALL);

     // Add the four columns we want to use to the
     CResString Col1 (hInst , IDS_COLUMN1);
     CResString Col2 (hInst , IDS_COLUMN2);
     CResString Col3 (hInst , IDS_COLUMN3);
     CResString Col4 (hInst , IDS_COLUMN4);

     InsertColumn (Col1 , 100 , 0);
     InsertColumn (Col2 , 100 , 1);
     InsertColumn (Col3 , 100 , 2);
     InsertColumn (Col4 , 200 , 3);
 }

 return (hwndListview != NULL);
}

/****************************************************************************/

