/****************************************************************************/

#include "nfi.h"

/*****************************************************************************
 * Name         - WinLibMain
 * Purpose      - This is the entry point to the program. It sets up the
 *                application and then starts the message loop that runs
 *                the program.
 * Arguments    - hInstance: Instance handle of the program
 *                lpCmdLine: Pointer to the string command line of the prog
 *                nCmdShow: Initial show state for the window
 * Return Value - FALSE if there is a problem
 *                WPARAM value of the Quit message that terminated the app on
 *                success.
 ****************************************************************************/
int winLibMain (HINSTANCE hInstance,
                LPSTR     lpCmdLine,
                int       nCmdShow)
{
 MSG    Msg;

 // Initialize application by setting up the main window.
 if (!InitApplication (hInstance, lpCmdLine , nCmdShow))
     return FALSE;

 // Load the accelerator table
 hAccelTable = (HACCEL) LoadAccelerators(hInstance, MAKEINTRESOURCE (IDA_ACCELERATORS));
 WINASSERT (hAccelTable != NULL);

 // Here we loop forever. The main loop will break as soon as a WM_QUIT message
 // received.
 while (1) {
     // Check to see if there is a message. If there is, then we want to
     // dispatch it. Otherwise, do the idle processing.
     if (PeekMessage (&Msg , NULL , 0 , 0 , PM_NOREMOVE)) {
         // If GetMessage returns 0, it means that a WM_QUIT message was
         // received. If that is the case, break out of this loop.
         if (!GetMessage (&Msg , NULL , 0 , 0))
             break;

         // It's not a WM_QUIT Message. Dispatch it in the usual manner
         if (!TranslateAccelerator (hwndMain , hAccelTable , &Msg)) {
             TranslateMessage (&Msg);
             DispatchMessage (&Msg);
         }
     } else {
         // There was no message. Call the Idle processing function to do
         // whatever we want to do when the program is idle. Once that is
         // Done we call WaitMessage. WaitMessage does not return until a
         // message is places in our queue again. Since the program state
         // cannot alter if we aren't getting any messages, there is no need
         // to bog the system calling IdleProcessing more than one time.
         IdleProcessing ();
         WaitMessage ();
     }
 }

 // Clean out the temporary directory, which may contain files we where asked
 // to temporarily extract to view.
 CleanTemporaryDirectory ();

 return Msg.wParam;
}

/****************************************************************************/

