/****************************************************************************/

#include "nfi.h"

/****************************************************************************/

// This is the message dispatching class for the move dialog box
class CMoveDialogDispatcher : public CDispatcher {
public:
    CMoveDialogDispatcher (handlerType Handler) : CDispatcher (Handler) {}

    LRESULT MsgInit (HWND, UINT, WPARAM, LPARAM);
    LRESULT MsgHelp (HWND, UINT, WPARAM, LPARAM);

    LRESULT CmdDone (HWND, WORD, WORD, HWND);

MESSAGE_LIST;
COMMAND_LIST;
};

START_MESSAGE_LIST (CMoveDialogDispatcher)
    MESSAGE (WM_HELP       , MsgHelp)
    MESSAGE (WM_INITDIALOG , MsgInit)
END_MESSAGE_LIST

START_COMMAND_LIST (CMoveDialogDispatcher)
    COMMAND (IDOK     , CmdDone)
    COMMAND (IDCANCEL , CmdDone)
END_COMMAND_LIST

/****************************************************************************/

static CMoveDialogDispatcher DialogDispatch (handlerNone);

/*****************************************************************************
 * Name         - MoveProc
 * Purpose      - This handles messages for the move dialog box by using it's
 *                dispatcher class.
 * Arguments    - Standard window procedure dialog boxes
 * Return Value - Depends on the message
 ****************************************************************************/
LRESULT CALLBACK MoveProc (HWND hDlg , UINT uMessage ,
                            WPARAM wParam, LPARAM lParam)
{
 return DialogDispatch.dispatchMessage (hDlg , uMessage , wParam , lParam);
}

/*****************************************************************************
 * Name         - CMoveDialogDispatcher::MsgInit
 * Purpose      - This handles the WM_INITDIALOG message.
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT CMoveDialogDispatcher::MsgInit (HWND hDlg , UINT /*uMessage*/ ,
                                        WPARAM /*wParam*/ , LPARAM /*lParam*/)
{
 SendMessage (GetDlgItem (hDlg , IDOK) , BM_SETIMAGE , (WPARAM) IMAGE_BITMAP , (LPARAM) hOKBitmap);
 SendMessage (GetDlgItem (hDlg , IDCANCEL) , BM_SETIMAGE , (WPARAM) IMAGE_BITMAP , (LPARAM) hCANCELBitmap);

 // Add all the path info to the dialog control.
 HWND hCtrl = GetDlgItem (hDlg , IDC_MOVEDIRCOMBO);

 // This recurses, adding all of the paths to the listbox portion of the
 // combobox.
 AddPathToComboBox (NFIPathInfo , hCtrl);

 SendMessage (hCtrl , CB_SETCURSEL , 0 , 0);

 // Tell the text control of the combo box how big it should let a person
 // type
 SendMessage (hCtrl , CB_LIMITTEXT , MAX_PATH - 1 , 0);

 // Center the dialog over the application window
 CenterWindow (hDlg , GetWindow (hDlg , GW_OWNER));

 return TRUE;
}

/*****************************************************************************
 * Name         - CMoveDialogDispatcher::MsgHelp
 * Purpose      - This handles the WM_HELP message.
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT CMoveDialogDispatcher::MsgHelp (HWND hDlg , UINT /*uMessage*/ ,
                                        WPARAM /*wParam*/ , LPARAM lParam)
{
 static DWORD HelpID[] = {
     IDC_MOVEDIRCOMBO  , IDC_MOVEDIRCOMBO  ,
     IDOK              , IDC_MOVEOK        ,
     IDCANCEL          , IDC_MOVECANCEL    ,
     0                 , 0 ,
 };

 char HelpFile[MAX_PATH];
 GetHelpFileName (HelpFile);

 HELPINFO hi = *(LPHELPINFO) lParam;

 if (hi.iContextType == HELPINFO_WINDOW &&
     hDlg != hi.hItemHandle) {
         WinHelp ((HWND) hi.hItemHandle ,
                  HelpFile ,
                  HELP_WM_HELP ,
                  (DWORD) (LPVOID) HelpID);
 }

 return TRUE;
}

/*****************************************************************************
 * Name         - MoveSelectedFilesToDir
 * Purpose      - This scans through the ListView finding every item that
 *                is selected. For each one, it alters the path information
 *                so that it conforms to the new path which has been passed
 *                in. This works for directory informatuon loaded from the
 *                NFI file, as well as new files waiting to be added.
 * Arguments    - hDLg: Handle to the dialog box that is the parent
 *                NewDir: New directory to put the selected files into
 * Return Value - None
 ****************************************************************************/
void MoveSelectedFilesToDir (HWND hDlg , char *NewDir)
{
 WINASSERT (hDlg);
 WINASSERT (NewDir);

 LONG ItemID = -1;
 LV_ITEM lvi;
 int ItemCount;

 // Find out how many selected items. If there are none, return.
 ItemCount = ListView_GetSelectedCount (hwndListview);
 if (ItemCount == 0)
     return;

 // Make the progress dialog box
 WINASSERT (hwndProgressDlg == NULL);
 CreateDialogParam (hInst, MAKEINTRESOURCE (IDD_PROGRESSDLG) , hDlg , (DLGPROC) ProgressProc , DELETE_PROGRESS);

 CResString Status (hInst , IDS_MOVINGFILES);
 SetWindowText (hwndTextCtrl , Status);

 // Turn off redraw for the control so we can fiddle with the icons without
 // it going nuts.
 SendMessage (hwndListview , WM_SETREDRAW , (WPARAM) FALSE , 0);

 // Keep searching for selected items until we get them all
 int ThisItem = 0;
 WPARAM Pct;
 while ((ItemID = ListView_GetNextItem (hwndListview , ItemID , LVNI_SELECTED)) != -1) {
    // Express an interest in receivng the image index and lParam value of
    // the selected item
    lvi.mask = LVIF_IMAGE | LVIF_PARAM;
    lvi.iItem = ItemID;
    lvi.iSubItem = 0;

    // Get the information
    ListView_GetItem (hwndListview , &lvi);

    // Depending on the image index, lParam is either an NFI Directory Entry
    // or the record of a file waiting to be added.
    if (lvi.iImage == 0) {
        // lParam is a directory entry
        NFIDirEntry *Dir = (NFIDirEntry *) lvi.lParam;

        // First, we need to split the path information for this node into
        // a file and a directory.
        char Buffer[_MAX_PATH + 10];
        char Path[_MAX_DIR];
        char FName[_MAX_FNAME];
        char Ext[_MAX_EXT];

        // Split the filename we got into it's parts,
        _splitpath  (Dir->Name , NULL , Path , FName , Ext);

        // Change the path
        strcpy (Path , NewDir);

        // Stick the filename back together and reset it in the directory
        // entry
        _makepath (Buffer , NULL , Path , FName , Ext);
        strcpy (Dir->Name , Buffer);

        ListView_SetItemText (hwndListview , ItemID , 3 , NewDir);
    } else {
        // lParam is an added file entry
        AddedFile *File = (AddedFile *) lvi.lParam;

        // To change the directory, all we have to do is change the node's
        // Path information to the given dir. We deallocate the current path
        // then reallocate a new one.
        delete[] File->InternalPath;
        File->InternalPath = new char[strlen (NewDir) + 1];
        strcpy (File->InternalPath , NewDir);

        ListView_SetItemText (hwndListview , ItemID , 3 , NewDir);
    }

     if (ThisItem == ItemCount)
         Pct = 100;
     else
         Pct = (WPARAM) (((float) ThisItem / (float) ItemCount) * 100.0);

     ThisItem++;
     SendMessage (hwndProgressCtrl , PBM_SETPOS , Pct , 0);

     PumpMessages ();
 }

 // Now turn redraw back on and get it to repaint
 SendMessage (hwndListview , WM_SETREDRAW , (WPARAM) TRUE , 0);
 InvalidateRect (hwndListview , NULL , FALSE);
 UpdateWindow (hwndListview);

 // Get rid of the progress dialog
 DestroyWindow (hwndProgressDlg);

 SetFileStatus (TRUE);
}

/*****************************************************************************
 * Name         - CMoveDialogDispatcher::CmdDone
 * Purpose      - This function handles command messages for the button and
 *                the standard windows close controls.
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT CMoveDialogDispatcher::CmdDone (HWND hDlg, WORD wCommand,
                                         WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 if (wCommand == IDOK) {
     char MoveToDir[MAX_PATH];
     GetWindowText (GetDlgItem (hDlg , IDC_MOVEDIRCOMBO) ,
                    MoveToDir ,
                    MAX_PATH);

     // If the string is empty, make it the root string
     if (MoveToDir[0] == '\0')
         strcpy (MoveToDir , "/");

     // Convert slashes
     char *Temp = MoveToDir;
     while (*Temp) {
         if (*Temp == '\\')
             *Temp = '/';
         Temp++;
     }

     // Make sure it has a trailing slash and a leading slash
     if (MoveToDir[0] != '/') {
         char Temp[MAX_PATH] = "/";
         strcat (Temp , MoveToDir);
         strncpy (MoveToDir , Temp , MAX_PATH);
         MoveToDir[MAX_PATH - 2] = '\0';
     }
     if (MoveToDir[strlen (MoveToDir) - 1] != '/')
         strcat (MoveToDir , "/");

     NFIPathInfo = AddPathToTree (NFIPathInfo , MoveToDir);

     MoveSelectedFilesToDir (hDlg , MoveToDir);
 }

 // End the dialog
 EndDialog (hDlg , wCommand);
 return TRUE;
}

/****************************************************************************/

