/****************************************************************************/

#include "nfi.h"

/****************************************************************************/

// This is the class which handles and dispatches messages for the main window
// in the application.
class CMainWndDispatcher : public CDispatcher {
 public:
     CMainWndDispatcher (handlerType Handler) : CDispatcher (Handler) {}

     LRESULT MsgSize         (HWND , UINT , WPARAM , LPARAM);
     LRESULT MsgDestroy      (HWND , UINT , WPARAM , LPARAM);
     LRESULT MsgClose        (HWND , UINT , WPARAM , LPARAM);
     LRESULT MsgNotify       (HWND , UINT , WPARAM , LPARAM);
     LRESULT MsgTrackMenu    (HWND , UINT , WPARAM , LPARAM);
     LRESULT MsgMenuEnable   (HWND , UINT , WPARAM , LPARAM);

     LRESULT CmdNew     (HWND , WORD , WORD , HWND);
     LRESULT CmdOpen    (HWND , WORD , WORD , HWND);
     LRESULT CmdSave    (HWND , WORD , WORD , HWND);
     LRESULT CmdSaveAs  (HWND , WORD , WORD , HWND);
     LRESULT CmdExit    (HWND , WORD , WORD , HWND);

     LRESULT CmdAddFiles     (HWND , WORD , WORD , HWND);
     LRESULT CmdDeleteFiles  (HWND , WORD , WORD , HWND);
     LRESULT CmdExtractFiles (HWND , WORD , WORD , HWND);
     LRESULT CmdViewFiles    (HWND , WORD , WORD , HWND);
     LRESULT CmdMoveFiles    (HWND , WORD , WORD , HWND);

     LRESULT CmdChangeView (HWND , WORD , WORD , HWND);
     LRESULT CmdOptions    (HWND , WORD , WORD , HWND);

     LRESULT CmdHelpContents (HWND , WORD , WORD , HWND);
     LRESULT CmdAbout        (HWND , WORD , WORD , HWND);

     LRESULT CmdLabelEdit (HWND , WORD , WORD , HWND);

 MESSAGE_LIST;
 COMMAND_LIST;
};

START_MESSAGE_LIST (CMainWndDispatcher)
    MESSAGE (WM_INITMENUPOPUP , MsgMenuEnable)
    MESSAGE (WM_NOTIFY        , MsgNotify)
    MESSAGE (WM_MENUSELECT    , MsgTrackMenu)
    MESSAGE (WM_DESTROY       , MsgDestroy)
    MESSAGE (WM_CLOSE         , MsgClose)
    MESSAGE (WM_SIZE          , MsgSize)
END_MESSAGE_LIST

START_COMMAND_LIST (CMainWndDispatcher)
    COMMAND (IDM_FILENEW      , CmdNew)
    COMMAND (IDM_FILEOPEN     , CmdOpen)
    COMMAND (IDM_FILESAVE     , CmdSave)
    COMMAND (IDM_FILESAVEAS   , CmdSaveAs)
    COMMAND (IDM_FILEEXIT     , CmdExit)

    COMMAND (IDM_ADDFILES     , CmdAddFiles)
    COMMAND (IDM_DELETEFILES  , CmdDeleteFiles)
    COMMAND (IDM_EXTRACTFILES , CmdExtractFiles)
    COMMAND (IDM_VIEWFILES    , CmdViewFiles)
    COMMAND (IDM_MOVEFILES    , CmdMoveFiles)

    COMMAND (IDM_BYNAME       , CmdChangeView)
    COMMAND (IDM_BYTYPE       , CmdChangeView)
    COMMAND (IDM_BYPATH       , CmdChangeView)
    COMMAND (IDM_BYSIZE       , CmdChangeView)
    COMMAND (IDM_BYOFFSET     , CmdChangeView)
    COMMAND (IDM_OPTIONS      , CmdOptions)

    COMMAND (IDM_HELPCONTENTS , CmdHelpContents)
    COMMAND (IDM_ABOUT        , CmdAbout)

    COMMAND (IDM_LABELEDIT    , CmdLabelEdit)
END_COMMAND_LIST

/****************************************************************************/

static CMainWndDispatcher MainWindowDispatch (handlerWindow);
static LONG CurrentSortItem;

/*****************************************************************************
 * Name         - WndProc
 * Purpose      - This handles messages sent to the main window. It uses the
 *                windows dispatching class to do it's job.
 * Arguments    - Standard window procedure parameters
 * Return Value - Depends on the message processes
 ****************************************************************************/
LRESULT CALLBACK WndProc (HWND hWnd , UINT uMessage,
                          WPARAM wParam , LPARAM lParam)
{
 // Ignore all command messages if there is a progress dialog box visible.
 if (hwndProgressDlg && uMessage == WM_COMMAND)
     return 0;

 return MainWindowDispatch.dispatchMessage (hWnd, uMessage, wParam, lParam);
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::MsgMenuEnable
 * Purpose      - This handles the WM_INITMENUPOPUP menu. We get this message
 *                before a popup menu is displayed by windows. We handle this
 *                to give us a chance to enable/disable items based on the
 *                program state. Note that the toolbar buttons are enabled
 *                and disabled in the IdleProcessing function in the file
 *                MISC.CPP, since they are displayed all the time.
 * Arguments    - hWnd: Handle of the window that's getting the message
 *                wParam: Handle to the menu being opened
 * Return Value - 1 if we process the message, 0 if not
 ****************************************************************************/
LRESULT CMainWndDispatcher::MsgMenuEnable (HWND hWnd , UINT /*uMessage*/ ,
                                           WPARAM wParam , LPARAM /*lParam*/)
{
 // Is the file menu opening? If so do command enabling for it
 if ((HMENU) wParam == GetSubMenu (GetMenu (hWnd) , FILEMENUPOS)) {
     UINT mf = FileIsDirty ? MF_ENABLED : MF_GRAYED;

     EnableMenuItem ((HMENU) wParam , IDM_FILESAVE , mf);

     return 0;
 }

 // Is the action menu opening? If so do command enabling for it
 if ((HMENU) wParam == GetSubMenu (GetMenu (hWnd) , ACTIONMENUPOS)) {
     UINT mf1 = (ListView_GetSelectedCount (hwndListview) > 0) ? MF_ENABLED : MF_GRAYED;
     UINT mf2 = (FilesInNFI + FilesToAdd != 0) ? MF_ENABLED : MF_GRAYED;

     EnableMenuItem ((HMENU) wParam , IDM_DELETEFILES , mf1);
     EnableMenuItem ((HMENU) wParam , IDM_MOVEFILES , mf1);
     EnableMenuItem ((HMENU) wParam , IDM_VIEWFILES , mf1);
     EnableMenuItem ((HMENU) wParam , IDM_EXTRACTFILES , mf2);
     return 0;
 }

 // Is the view menu opening? If so do command enabling for it
 if ((HMENU) wParam == GetSubMenu (GetMenu (hWnd) , VIEWMENUPOS)) {
     UINT mf = (FilesInNFI + FilesToAdd != 0) ? MF_ENABLED : MF_GRAYED;

     EnableMenuItem ((HMENU) wParam , IDM_BYNAME , mf);
     EnableMenuItem ((HMENU) wParam , IDM_BYTYPE , mf);
     EnableMenuItem ((HMENU) wParam , IDM_BYPATH , mf);
     EnableMenuItem ((HMENU) wParam , IDM_BYSIZE , mf);
     EnableMenuItem ((HMENU) wParam , IDM_BYOFFSET , mf);

     return 0;
 }

 // Is the help menu opening? If so, do command enabling for it
 if ((HMENU) wParam == GetSubMenu (GetMenu (hWnd) , HELPMENUPOS)) {
     return 0;
 }

 // Pass the message if we didn't get it
 return 1;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::MsgNotify
 * Purpose      - This handles the WM_NOTIFY message that is sent to the main
 *                window by the toolbar whenever it needs a string of text
 *                to display for a tooltip, or when the user edits a list
 *                item's label
 * Arguments    - lParam: Pointer to the struct that tells us what tooltip to
 *                        provide
 * Return Value - Depends on the notification message handled
 ****************************************************************************/
LRESULT CMainWndDispatcher::MsgNotify (HWND hWnd , UINT /*uMessage*/ ,
                                       WPARAM /*wParam*/ , LPARAM lParam)
{
 LPTOOLTIPTEXT lpToolTipText;
 LV_DISPINFO   *lpDispInfo;
 NM_LISTVIEW   *lpClickInfo;
 LPNMHDR       NormalNotify;
 static char   szBuffer[65];

 // Get the pointer to the notification structure.
 lpToolTipText = (LPTOOLTIPTEXT) lParam;

 // Is the control telling us that we need tooltip text?
 if (lpToolTipText->hdr.code == TTN_NEEDTEXT) {
     // Yes, get the ID of the control.
     UINT ID = lpToolTipText->hdr.idFrom;

     CResString ToolTip (hInst , ID , szBuffer , sizeof (szBuffer));

     // Set the tooltip text to our string.
     lpToolTipText->lpszText = szBuffer;
     return 0;
 }

 // Get a pointer to the notification structure?
 lpDispInfo = (LV_DISPINFO *) lParam;

 // Is the control telling us that the user has ended editing a list item
 // label?
 if (lpDispInfo->hdr.code == LVN_ENDLABELEDIT) {
     // Was the label actually changed?
     if (lpDispInfo->item.pszText != NULL) {
         // THe label was changed. We now tweak up the mast member of the
         // structure and ask the listview to give is more info on it. This
         // is because by default it doesn't mention the image index but we
         // need to know that to know what sort of file we have.
         LV_ITEM ThisItem;
         ThisItem.mask = LVIF_IMAGE | LVIF_PARAM;
         ThisItem.iItem = lpDispInfo->item.iItem;
         ThisItem.iSubItem = 0;

         ListView_GetItem (hwndListview , &ThisItem);

         if (ThisItem.iImage == 0) {
             // This is a loaded directory entry.
             NFIDirEntry *Entry = (NFIDirEntry *) ThisItem.lParam;

             // In order to update the name we need to append the path info
             // to the entry's name, then the new entry text.
             ListView_GetItemText (hwndListview , ThisItem.iItem , 3 ,
                                   Entry->Name , sizeof (Entry->Name));
             strcat (Entry->Name , lpDispInfo->item.pszText);
         } else {
             // This is an entry waiting to be added
             AddedFile *Entry = (AddedFile *) ThisItem.lParam;

             // Update the name portion of this entry. To do that we get rid
             // of the old internal name, allocate a new one and then copy
             // over.
             delete[] Entry->InternalName;
             Entry->InternalName = new char[strlen (lpDispInfo->item.pszText) + 1];
             strcpy (Entry->InternalName , lpDispInfo->item.pszText);
         }

         // Tell the control that we want to keep the change
         // We also set the file dirty flag, because the file has changed.
         FileIsDirty = TRUE;
         return TRUE;
     }
 }

 // Get a pointer to the notification structure?
 lpClickInfo = (NM_LISTVIEW *) lParam;

 // Is the control telling us that the user has clicked on a column header?
 if (lpClickInfo->hdr.code == LVN_COLUMNCLICK) {
     // If there is a progress dialog box in existance and we get here, chances
     // are very good that it is a sort progress dialog box. For some reason
     // if a header "button" is in the down position when a progress dialog
     // box is brought into existance, the listview control is disabled but
     // the associated header control is not, until the focus is shifted from
     // it. This makes it possible to click on a column to start sorting and
     // then click again on another column, which is not good. So here we check
     // to see if there is a progress dialog at the moment, and if so, do
     // nothing.
     if (hwndProgressDlg)
         return 0;

     // Make a command for this item.
     WORD wCommand = lpClickInfo->iSubItem;

     switch (wCommand) {
         case 0: wCommand = IDM_BYNAME; break;
         case 1: wCommand = IDM_BYSIZE; break;
         case 2: wCommand = IDM_BYOFFSET; break;
         case 3: wCommand = IDM_BYPATH; break;
    }

     // Call the sort handler to do the job
     CmdChangeView (hWnd , wCommand , 0 , NULL);

     return 0;
 }


 NormalNotify = (LPNMHDR) lParam;

 // Is this a double click message?
 if (NormalNotify->code == NM_DBLCLK) {
     // Is it from the listview? If so fake a view file message.
     if (NormalNotify->hwndFrom == hwndListview)
         CmdViewFiles (hWnd , 0 , 0 , 0);
 }

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::MsgTrackMenu
 * Purpose      - This function is called to track the current selection in
 *                the main window's menu as the user is selecting it. It
 *                determines the command ID of the selected menu item and
 *                attempts to load in the resource string with the same ID.
 *                It is then displayed in the status bar
 * Arguments    - Look in the help for WM_MENUSELECT message
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::MsgTrackMenu (HWND /*hWnd*/ , UINT /*uMessage*/,
                                          WPARAM wParam , LPARAM lParam)
{
 WINASSERT (hwndStatusbar);
 static char szBuffer[65];
 INT    nStringID = 0;
 UINT   fuFlags = GET_WM_MENUSELECT_FLAGS (wParam , lParam) & 0xffff;
 UINT   uCmd    = GET_WM_MENUSELECT_CMD (wParam , lParam);
 HMENU  hMenu   = GET_WM_MENUSELECT_HMENU (wParam , lParam);

 // These are a list of the ID values of all of the top level menu items. This
 // is used in statusbar tracking of the current menu item.
 static UINT idPopup[] = {
     IDM_FILEMENU,
     IDM_ACTIONMENU,
     IDM_VIEWMENU,
     IDM_HELPMENU,
 };

 // Reset the buffer.
 szBuffer[0] = 0;

 // Menu has been Closed
 if (fuFlags == 0xffff && hMenu == NULL)
     nStringID = 0;
 else
     // Ignore menu seperators
     if (fuFlags & MFT_SEPARATOR)
         nStringID = 0;
     else
         // It's a popup menu
         if (fuFlags & MF_POPUP) {
             // It's the system menu
             if (fuFlags & MF_SYSMENU)
                 nStringID = IDM_SYSMENU;
             else {
                 // Get the string ID for the popup menu
                 nStringID = ((uCmd < sizeof (idPopup) / sizeof(idPopup[0])) ?
                                 idPopup[uCmd] : 0);
             }
         } else {
             // Must be a command item
             // String ID is the same as the command ID
             nStringID = uCmd;
         }

 // If we have an ID, load the string that goes with it. Set the status bar
 // to simple mode with popout to display it. If there isn't an ID then we
 // put the status bar back to the way it should be.
 if (nStringID != 0) {
     CResString TopicStr (hInst , nStringID , szBuffer , sizeof (szBuffer));

     // Set the statusbar to simple mode (the entire bar becomes one text
     // area with an ID of 255) and display our string in that text area
     // using a popout style.
     SendMessage (hwndStatusbar , SB_SIMPLE , (WPARAM) TRUE , 0);
     UpdateStatusbar (szBuffer , STATBAR_PANEALL , SBT_POPOUT);
 } else {
     // There isn't anything to display, so set the statusbar back to the
     // normal mode. The status panes will return to their previous state
     // and contents.
     SendMessage (hwndStatusbar , SB_SIMPLE , (WPARAM) FALSE , 0);
 }

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::MsgDestroy
 * Purpose      - This handles the WM_DESTROY message sent to the window. It
 *                posts a quit message to the main window, which causes the
 *                message loop to terminte and make the application terminate.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::MsgDestroy (HWND hWnd , UINT /*uMessage*/,
                                        WPARAM /*wParam*/ , LPARAM /*lParam*/)
{
 if (hToolbarBitmap)
     DeleteObject (hToolbarBitmap);

 if (hOKBitmap)
     DeleteObject (hOKBitmap);

 if (hCANCELBitmap)
     DeleteObject (hCANCELBitmap);

 if (hADDFILEBitmap)
     DeleteObject (hADDFILEBitmap);

 if (hADDDIRBitmap)
     DeleteObject (hADDDIRBitmap);

 if (hREMOVEBitmap)
     DeleteObject (hREMOVEBitmap);

 if (hBROWSEBitmap)
     DeleteObject (hBROWSEBitmap);

 // Save our window placement if we are supposed to
 if (Setup.SaveWindowPos)
     SaveMainWindowPosition ();

 // If we are supposed to reopen the last open file, we will now make a copy
 // of the current filename into the Setup struct, then save it into the
 // registry.
 if (Setup.ReopenLast) {
     strcpy (Setup.LastFileName , Filename);
     SaveSetupStructToRegistry (Setup);
 }

 // Get rid of the NFI Path information
 DeletePathTree ();

 // Close the help application
 char HelpFileName[MAX_PATH];
 GetHelpFileName (HelpFileName);
 WinHelp (hWnd , HelpFileName , HELP_QUIT , 0);

 PostQuitMessage (0);
 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::MsgClose
 * Purpose      - This handles the WM_CLOSE message for the main window, which
 *                is generated whenever the user chooses to close the program
 *                using the Exit menu item, presses Alt-F4 or uses the window
 *                caption control to close the window. It checks to see if
 *                there is unsaved work, and if so asks the user what they
 *                want to do before quitting.
 * Arguments    - hWnd: Window being closed
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::MsgClose (HWND hWnd , UINT /*uMessage*/,
                                      WPARAM /*wParam*/ , LPARAM /*lParam*/)
{
 // Do nothing if there is a progress dialog box present
 if (hwndProgressDlg)
     return 0;

 // Ask the user if they want to save if the file isn't saved yet.
 if (FileIsDirty) {
     int Ret = PromptToSave (IDS_CONFIRM , IDS_QUITPROMPT);

     if (Ret == IDCANCEL)
         return 0;

     if (Ret == IDYES)
         DoSave ();
 }

 // Call the DoNew () routine, which will release all data strucutures that
 // may be pending for any open file.
 DoNew ();

 DestroyWindow (hWnd);
 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::MsgSize
 * Purpose      - Handles the WM_SIZE message sent to the main window. We
 *                take this opportunity to resize the child windows to the
 *                new parent window size
 * Arguments    - hWnd: Window changing size
 * Return Value - 0;
 ****************************************************************************/
LRESULT CMainWndDispatcher::MsgSize (HWND hWnd , UINT uMessage,
                                     WPARAM wParam , LPARAM lParam)
{
 WINASSERT (hwndToolbar);
 WINASSERT (hwndStatusbar);

 SendMessage (hwndToolbar , uMessage , wParam , lParam);
 SendMessage (hwndStatusbar , uMessage , wParam , lParam);

 // Reposition the panes in the status bar according to the new size
 InitializeStatusbar (hWnd);

 // Position the list view window between the toolbar and the statusbar.
 if (wParam != SIZE_MINIMIZED) {
     RECT rc, rcClient;

     // First get the size of the client window in general.
     GetClientRect (hWnd , &rcClient);

     // Next, we get the size of the toolbar and adjust the top of the client
     // rect to take that into account.
     GetWindowRect (hwndToolbar , &rc);
     ScreenToClient (hWnd , (LPPOINT) &rc.right);
     rcClient.top = rc.bottom;

     // Next, get the size of the toolbar and adjust the bottom of the
     // client rectangle to take that into account as well.
     GetWindowRect (hwndStatusbar , &rc);
     ScreenToClient (hWnd , (LPPOINT) &rc.left);
     rcClient.bottom = rc.top;

     // With our newfound rectangle, size the listview window to be the
     // part of the client between the two bars.
     MoveWindow (hwndListview , rcClient.left , rcClient.top ,
                 rcClient.right - rcClient.left ,
                 rcClient.bottom - rcClient.top , TRUE);
 }

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdNew
 * Purpose      - Handles the File|New menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdNew (HWND /*hWnd*/, WORD /*wCommand*/,
                                    WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 if (FileIsDirty) {
     int Ret = PromptToSave (IDS_CONFIRM , IDS_NEWPROMPT);

     if (Ret == IDCANCEL)
         return 0;

     if (Ret == IDYES)
         DoSave ();
 }

 DoNew ();
 strcpy (Filename , "");
 MergeFilename (hwndMain , szTitle , Filename);
 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdOpen
 * Purpose      - Handles the File|Open menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdOpen (HWND /*hWnd*/, WORD /*wCommand*/,
                                    WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 if (FileIsDirty) {
     int Ret = PromptToSave (IDS_CONFIRM , IDS_OPENPROMPT);

     if (Ret == IDCANCEL)
         return 0;

     if (Ret == IDYES)
         DoSave ();
 }

 // Get Filename
 DoOpen (TRUE);
 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdSave
 * Purpose      - Handles the File|Save menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdSave (HWND /*hWnd*/, WORD /*wCommand*/,
                                     WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 if (Filename[0] == '\0')
     DoSaveAs ();
 else
     DoSave ();

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdSaveAs
 * Purpose      - Handles the File|Save As menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdSaveAs (HWND /*hWnd*/, WORD /*wCommand*/,
                                       WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 DoSaveAs ();
 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdExit
 * Purpose      - This function is called in response to the user choosing the
 *                exit menu item from the file menu. It simply tells the
 *                window to close.
 * Arguments    - hWnd: Window being closed
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdExit (HWND hWnd, WORD /*wCommand*/,
                                     WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 PostMessage (hWnd , WM_CLOSE , 0 , 0);
 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdAddFiles
 * Purpose      - Handles the Actions|Add Files menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdAddFiles (HWND hWnd, WORD /*wCommand*/,
                                         WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 if (DialogBox (hInst, MAKEINTRESOURCE (IDD_ADDDLG) , hWnd , (DLGPROC) AddProc) == IDOK) {
    UpdateWindow (hwndToolbar);
    // Make the progress dialog box
    WINASSERT (hwndProgressDlg == NULL);
    CreateDialogParam (hInst, MAKEINTRESOURCE (IDD_PROGRESSDLG) , hWnd , (DLGPROC) ProgressProc , ADD_PROGRESS);

    CResString Status (hInst , IDS_ADDINGNEWFILES);
    SetWindowText (hwndTextCtrl , Status);

    // Add the selected files to the global list of files to be added.
    UpdateAddedFileList ();
    DestroyWindow (hwndAddListBox);

    // Get rid of the progress dialog
    DestroyWindow (hwndProgressDlg);
 }

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdDeleteFiles
 * Purpose      - Handles the Actions|Delete Files menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdDeleteFiles (HWND hWnd , WORD /*wCommand*/,
                                            WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 LONG ItemID = -1;
 LV_ITEM lvi;
 int ItemCount;

 // Find out how many selected items. If there are none, return.
 ItemCount = ListView_GetSelectedCount (hwndListview);
 if (ItemCount == 0)
     return 0;

 // Prompt the user to see if they are sure they want to delete.
 CResString Title  (hInst , IDS_CONFIRM);
 CResString Prompt (hInst , IDS_DELETEPROMPT);

 if (MessageBox (hWnd , Prompt , Title , MB_YESNO | MB_ICONQUESTION) == IDNO)
     return 0;

 // Make the progress dialog box
 WINASSERT (hwndProgressDlg == NULL);
 CreateDialogParam (hInst, MAKEINTRESOURCE (IDD_PROGRESSDLG) , hwndMain , (DLGPROC) ProgressProc , DELETE_PROGRESS);

 CResString Status (hInst , IDS_DELETINGFILES);
 SetWindowText (hwndTextCtrl , Status);

 // Turn off redraw for the control so we can fiddle with the icons without
 // it going nuts.
 SendMessage (hwndListview , WM_SETREDRAW , (WPARAM) FALSE , 0);

 // Keep looking for selected items. We start by specifying an item index of
 // -1 as the item to start searching from. After we find one (if we do)
 // we delete it, and then back the index up 1 so that it searches from that
 // item.
 int ThisItem = 0;
 WPARAM Pct;
 while ((ItemID = ListView_GetNextItem (hwndListview , ItemID , LVNI_SELECTED)) != -1) {
     // Get information about the item that we are deleting.
     lvi.mask = LVIF_IMAGE | LVIF_PARAM;
     lvi.iItem = ItemID;
     lvi.iSubItem = 0;

     // Get the information for this item so that we can remove it from the
     // proper linked list
     ListView_GetItem (hwndListview , &lvi);

     // Depending on the image index, lParam is either an NFI Directory Entry
     // or the record of a file waiting to be added.
     if (lvi.iImage == 0) {
        // lParam is a directory entry
        NFIDirEntry *Dir = (NFIDirEntry *) lvi.lParam;
        UnlinkDirEntry (Dir);
        FilesInNFI--;
     } else {
        // lParam is an added file entry
        AddedFile *File = (AddedFile *) lvi.lParam;
        UnlinkAddEntry (File);
        FilesToAdd--;
     }

     // Now delete this item from the listview
     ListView_DeleteItem (hwndListview , ItemID);
     ItemID--;

     if (ThisItem == ItemCount)
         Pct = 100;
     else
         Pct = (WPARAM) (((float) ThisItem / (float) ItemCount) * 100.0);

     ThisItem++;
     SendMessage (hwndProgressCtrl , PBM_SETPOS , Pct , 0);

     PumpMessages ();
 }

 // Now turn redraw back on and get it to repaint
 SendMessage (hwndListview , WM_SETREDRAW , (WPARAM) TRUE , 0);
 InvalidateRect (hwndListview , NULL , FALSE);
 UpdateWindow (hwndListview);
 SetFileStatus (TRUE);
 UpdateFileCount ();

 // Get rid of the progress dialog
 DestroyWindow (hwndProgressDlg);

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdExtractFiles
 * Purpose      - Handles the Actions|Extract Files menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdExtractFiles (HWND hWnd, WORD /*wCommand*/,
                                             WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 if (DialogBox (hInst, MAKEINTRESOURCE (IDD_EXTRACTDLG) , hWnd , (DLGPROC) ExtractProc) == IDOK)
     DoExtraction ();
 return 0;
}

/*****************************************************************************
 * Name         - ViewErrorBox
 * Purpose      - This displays a message box for the error specified in code.
 *                This is used when the program tries to execute another
 *                program in response to viewing a file.
 * Arguments    - Code: Error code
 *                Name: Filename that caused the error
 * Return Value - None
 ****************************************************************************/
void ViewErrorBox (int Code , char *Name)
{
 WINASSERT (Name);
 int MsgId;

 switch (Code) {
     case 0:                      MsgId = IDS_VIEWERRORNOMEM; break;
     case SE_ERR_OOM:             MsgId = IDS_VIEWERRORNOMEM; break;
     case ERROR_FILE_NOT_FOUND:   MsgId = IDS_VIEWERRORNOFILE; break;
     case ERROR_PATH_NOT_FOUND:   MsgId = IDS_VIEWERRORNOPATH; break;
     case ERROR_BAD_FORMAT:       MsgId = IDS_VIEWERRORBADFORM; break;
     case SE_ERR_ACCESSDENIED:    MsgId = IDS_VIEWERRORNOACCESS; break;
     case SE_ERR_SHARE:           MsgId = IDS_VIEWERRORNOACCESS; break;
     case SE_ERR_ASSOCINCOMPLETE: MsgId = IDS_VIEWERRORBADASSOC; break;
     case SE_ERR_NOASSOC:         MsgId = IDS_VIEWERRORBADASSOC; break;
     case SE_ERR_DDEBUSY:         MsgId = IDS_VIEWERRORBUSYDDE; break;
     case SE_ERR_DDEFAIL:         MsgId = IDS_VIEWERRORFAILDDE; break;
     case SE_ERR_DDETIMEOUT:      MsgId = IDS_VIEWERRORTIMEDDE; break;
     case SE_ERR_DLLNOTFOUND:     MsgId = IDS_VIEWERRORNODLL; break;
 }

 CResString Title (hInst , IDS_VIEWERRORTITLE);
 CResString Msg   (hInst , MsgId);
 char Buffer[MAX_PATH];
 wsprintf (Buffer , Msg , Name);

 MessageBox (hwndMain , Buffer , Title , MB_OK | MB_ICONERROR);

}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdViewFiles
 * Purpose      - Handles the Actions|View Files menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdViewFiles (HWND hWnd , WORD /*wCommand*/,
                                         WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 // Find the first selected item.
 LONG ItemID = ListView_GetNextItem (hwndListview , -1 , LVNI_SELECTED);
 LV_ITEM lvi;
 char ExtractedName[MAX_PATH];
 char TempDirectory[MAX_PATH];

 // Get the temporary directory.
 GetTemporaryDirectory (TempDirectory);

 // Loop until there are no more selected items.
 while (ItemID != -1) {
     // Get information on the item we have at the moment
     lvi.iItem = ItemID;
     lvi.iSubItem = 0;
     lvi.mask = LVIF_IMAGE | LVIF_PARAM | LVIF_STATE;
     lvi.stateMask = LVNI_SELECTED;

     ListView_GetItem (hwndListview , &lvi);

     // Try to extract the entry.
     if (ExtractEntry (lvi , TRUE , ExtractedName)) {
         // It worked. Try to execute it.
         int Execed = (int) ShellExecute (hWnd , "open" , ExtractedName , "" ,
                                          TempDirectory , SW_SHOWNORMAL);

         if (Execed <= 32)
             ViewErrorBox (Execed , ExtractedName);
     }

     // Try to get the next item
     ItemID = ListView_GetNextItem (hwndListview , ItemID , LVNI_SELECTED);
 }

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdMoveFiles
 * Purpose      - Handles the Actions|Move Files menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdMoveFiles (HWND hWnd, WORD /*wCommand*/,
                                          WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 DialogBox (hInst, MAKEINTRESOURCE (IDD_MOVEDLG) , hWnd , (DLGPROC) MoveProc);

 return 0;
}

/*****************************************************************************
 * Name         - GetNodeName
 * Purpose      - Gets the name portion only out of the given node and puts it
 *                into buffer
 * Arguments    - lParam: Node to examine
 *                Image: Indicator of what sort of node we have
 *                Buffer: Place to put the name.
 * Return Value - None
 ****************************************************************************/
static void GetNodeName (LPARAM lParam , int Image , char *Buffer)
{
 WINASSERT (Buffer);

 if (Image == 1) {
     AddedFile *File = (AddedFile *) lParam;
     strcpy (Buffer , File->InternalName);
 } else {
     NFIDirEntry *Entry = (NFIDirEntry *) lParam;

     char Name[_MAX_FNAME];
     _splitpath (Entry->Name , NULL , NULL , Name , NULL);
     strcpy (Buffer , Name);
 }
}

/*****************************************************************************
 * Name         - GetNodeType
 * Purpose      - Gets the type portion only out of the given node and puts it
 *                into buffer
 * Arguments    - lParam: Node to examine
 *                Image: Indicator of what sort of node we have
 *                Buffer: Place to put the type.
 * Return Value - None
 ****************************************************************************/
inline static void GetNodeType (LPARAM lParam , int Image , char *Buffer)
{
 WINASSERT (Buffer);

 if (Image == 1) {
     AddedFile *File = (AddedFile *) lParam;
     char Ext[_MAX_EXT];
     _splitpath (File->InternalName , NULL , NULL , NULL , Ext);
     strcpy (Buffer , Ext);
 } else {
     NFIDirEntry *Entry = (NFIDirEntry *) lParam;

     char Ext[_MAX_EXT];
     _splitpath (Entry->Name , NULL , NULL , NULL , Ext);
     strcpy (Buffer , Ext);
 }
}

/*****************************************************************************
 * Name         - GetNodePath
 * Purpose      - Gets the path portion only out of the given node and puts it
 *                into buffer
 * Arguments    - lParam: Node to examine
 *                Image: Indicator of what sort of node we have
 *                Buffer: Place to put the path.
 * Return Value - None
 ****************************************************************************/
inline static void GetNodePath (LPARAM lParam , int Image , char *Buffer)
{
 WINASSERT (Buffer);

 if (Image == 1) {
     AddedFile *File = (AddedFile *) lParam;
     strcpy (Buffer , File->InternalPath);
 } else {
     NFIDirEntry *Entry = (NFIDirEntry *) lParam;

     char Path[_MAX_PATH];
     _splitpath (Entry->Name , NULL , Path , NULL , NULL);
     strcpy (Buffer , Path);
 }
}

/*****************************************************************************
 * Name         - GetNodeSize
 * Purpose      - Gets the size portion only out of the given node and puts it
 *                into Long
 * Arguments    - lParam: Node to examine
 *                Image: Indicator of what sort of node we have
 *                Long: Place to put the size.
 * Return Value - None
 ****************************************************************************/
inline static void GetNodeSize (LPARAM lParam , int Image , LONG &Long)
{
 if (Image == 1) {
     Long = 0;
 } else {
     NFIDirEntry *Entry = (NFIDirEntry *) lParam;
     Long = Entry->Length;
 }
}

/*****************************************************************************
 * Name         - GetNodeOffs
 * Purpose      - Gets the Offs portion only out of the given node and puts it
 *                into Long
 * Arguments    - lParam: Node to examine
 *                Image: Indicator of what sort of node we have
 *                Long: Place to put the Offs.
 * Return Value - None
 ****************************************************************************/
inline static void GetNodeOffs (LPARAM lParam , int Image , LONG &Long)
{
 if (Image == 1) {
     Long = 0;
 } else {
     NFIDirEntry *Entry = (NFIDirEntry *) lParam;
     Long = Entry->Offset;
 }
}

/*****************************************************************************
 * Name         - SortFunction
 * Purpose      - Called during sorting so the listbox can tell how to sort
 *                the items.
 * Arguments    - lParam1: lParam of the first item to be sorted
 *                lParam2: lParam of the second item to be sorted
 *                lParamSort: Command ID specifying how we want to sort.
 * Return Value - < 0 : lParam1 comes before lParam2
 *                0   : lParam1 == lParam2
 *                > 0 : lParam1 comes after lParam2;
 ****************************************************************************/
static int CALLBACK SortFunction (LPARAM lParam1 , LPARAM lParam2 , LPARAM lParamSort)
{
 LV_FINDINFO lvfi1;
 LV_FINDINFO lvfi2;

 // The search function passes us the two lParam values of the two items it
 // wants to sort. Unfortunately for us we have a bit of a problem with this.
 // Depending on if the item represents a loaded directory entry or one that
 // is waiting to be added, the pointers are to two different structures. We
 // can tell the difference by examining the image index, which tells us what
 // sort of node this is. So we first have to find the indexes of the two
 // ites we have, then get information on each of them to find out their
 // indexes. Then we can sort.

 // Search to find the two items which match the two paramters we where
 // given.
 lvfi1.flags = lvfi2.flags = LVFI_PARAM;
 lvfi1.lParam = lParam1;
 lvfi2.lParam = lParam2;

 int Item1 = ListView_FindItem (hwndListview , -1 , &lvfi1);
 int Item2 = ListView_FindItem (hwndListview , -1 , &lvfi2);

 WINASSERT (Item1 != -1 && Item2 != -1);

 // Now we know the indexes of the two passed in items. Get those actual
 // items (we care about the image index only).
 LV_ITEM lvi1;
 LV_ITEM lvi2;

 lvi1.mask = lvi2.mask = LVIF_IMAGE;
 lvi1.iSubItem = lvi2.iSubItem = 0;
 lvi1.iItem = Item1;
 lvi2.iItem = Item2;
 ListView_GetItem (hwndListview , &lvi1);
 ListView_GetItem (hwndListview , &lvi2);

 char Buffer1[MAX_PATH];
 char Buffer2[MAX_PATH];
 LONG Long1;
 LONG Long2;

 // By default assume we want to sort descendng.
 int  SortBy=lParamSort;
 BOOL Ascending=FALSE;

 // Set the sort ascending flag if needed.
 if (lParamSort >= 128) {
     SortBy = lParamSort - 128;
     Ascending = TRUE;
 }

 WPARAM Pct;
 LONG Total = (FilesInNFI + FilesToAdd);

 // The bad thing about using a progress control to show the sort progress
 // is that depending on the nature of the data we can never tell exactly how
 // many passes it will take to sort. So what we do is make this static
 // variable and increment that on every pass through the loop. After 20
 // passes we finally update the progress control. This keeps the control
 // looking like it is updating for the whole sort, even though it may not
 // max out by the time the job is done, at least the user knows that it has
 // not screwed up yet.
 static int TimeToUpdate=0;

 // Update the progress control
 if (CurrentSortItem == Total)
     Pct = 100;
 else
     Pct = (WPARAM) (((float) CurrentSortItem / (float) Total) * 100.0);

 TimeToUpdate++;
 if (TimeToUpdate == 20) {
     CurrentSortItem++;
     TimeToUpdate = 0;
 }
 SendMessage (hwndProgressCtrl , PBM_SETPOS , Pct , 0);

 // Pump waiting messages to keep things responsive.
 PumpMessages ();

 // Now we have all the info we need to do the sort. Depending on what we
 // are supposed to do, do something different
 switch (SortBy) {
     case 0:
         // Get the filename portion of the nodes only
         GetNodeName (lParam1 , lvi1.iImage , Buffer1);
         GetNodeName (lParam2 , lvi2.iImage , Buffer2);

         if (Ascending)
             return strcmp (Buffer1 , Buffer2);
         else
             return strcmp (Buffer2 , Buffer1);
     case 1:
         // Get the type portion of the nodes only
         GetNodeType (lParam1 , lvi1.iImage , Buffer1);
         GetNodeType (lParam2 , lvi2.iImage , Buffer2);

         if (Ascending)
             return strcmp (Buffer1 , Buffer2);
         else
             return strcmp (Buffer2 , Buffer1);

     case 2:
         // Get the path portion of the nodes only
         GetNodePath (lParam1 , lvi1.iImage , Buffer1);
         GetNodePath (lParam2 , lvi2.iImage , Buffer2);

         if (Ascending)
             return strcmp (Buffer1 , Buffer2);
         else
             return strcmp (Buffer2 , Buffer1);

     case 3:
         // Get the size portion of the nodes only
         GetNodeSize (lParam1 , lvi1.iImage , Long1);
         GetNodeSize (lParam2 , lvi2.iImage , Long2);

         if (Ascending)
             return Long1 - Long2;
         else
             return Long2 - Long1;

     case 4:
         // Get the offset portion of the nodes only
         GetNodeOffs (lParam1 , lvi1.iImage , Long1);
         GetNodeOffs (lParam2 , lvi2.iImage , Long2);

         if (Ascending)
             return Long1 - Long2;
         else
             return Long2 - Long1;
 }

 // We should never get here, but just in case we put this here to shut the
 // compiler up.
 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdChangeView
 * Purpose      - Handles all of the View|Sort by menu commands.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdChangeView (HWND hWnd , WORD wCommand,
                                           WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 WINASSERT (hWnd);

 // This static tracks an Ascending/Descending value for each of the 5
 // possible sort values. Every pass through the function we invert the setting
 // then use the new one. So these are intially set to the opposite of what we
 // really want.
 static BOOL Ascending[5]={FALSE, FALSE, FALSE, FALSE, FALSE};

 // If there are no files in the listview we have nothing to do.
 if (FilesInNFI + FilesToAdd == 0)
     return 0;

 // Determine the 0-4 numbr of what we want to sort by to pass to the sort
 // function.
 int SortBy = wCommand - IDM_BYNAME;

 // Toggle the ascending value of this item.
 Ascending[SortBy] = !Ascending[SortBy];

 // Add 128 to this value if we want to sort ascending.
 if (Ascending[SortBy])
     SortBy += 128;

 // Start the progress control at 0 (this is a global variable because we
 // can't tell when the sort function is called which is the first time).
 CurrentSortItem = 0;

 // Make the progress dialog box
 WINASSERT (hwndProgressDlg == NULL);
 CreateDialogParam (hInst, MAKEINTRESOURCE (IDD_PROGRESSDLG) , hWnd , (DLGPROC) ProgressProc , SORT_PROGRESS);

 CResString Status (hInst , IDS_SORTINGFILES);
 SetWindowText (hwndTextCtrl , Status);

 // Tell the Listview to sort itself.
 ListView_SortItems (hwndListview , SortFunction , SortBy);

 // Get rid of the progress dialog
 DestroyWindow (hwndProgressDlg);

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdOptions
 * Purpose      - Handles the View|Options menu command.
 * Arguments    - None Used
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdOptions (HWND hWnd , WORD /*wCommand*/,
                                        WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 DialogBox (hInst, MAKEINTRESOURCE (IDD_OPTIONDLG) , hWnd , (DLGPROC) OptionProc);
 return 0;
}


/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdHelpContents
 * Purpose      - This opens up the help file to the contents page
 * Arguments    - hWnd: Handle of the main window
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdHelpContents (HWND hWnd, WORD /*wCommand*/,
                                             WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 char HelpFileName[MAX_PATH];

 // Get the fully qualified help file name.
 GetHelpFileName (HelpFileName);

 WinHelp (hWnd , HelpFileName , HELP_FINDER , 0);

 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdAbout
 * Purpose      - This is called to display the about box when the user
 *                selects the about menu item
 * Arguments    - hWnd: Handle of the main window
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdAbout (HWND hWnd, WORD /*wCommand*/,
                                      WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 DialogBox (hInst, MAKEINTRESOURCE (IDD_ABOUTDLG) , hWnd , (DLGPROC) AboutProc);
 return 0;
}

/*****************************************************************************
 * Name         - CMainWndDispatcher::CmdLabelEdit
 * Purpose      - This is called to start editing of the currently focused
 *                listview item.
 * Arguments    - hWnd: Handle of the main window
 * Return Value - 0
 ****************************************************************************/
LRESULT CMainWndDispatcher::CmdLabelEdit (HWND /*hWnd*/, WORD /*wCommand*/,
                                          WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 // Get the focused item
 int ItemID = ListView_GetNextItem (hwndListview , -1 , LVNI_FOCUSED);

 // If there isn't a focused item (highly unlikely) retyrn
 if (ItemID == -1)
     throw 0;

 // Tell the listview control to start in-place editing of this item.
 ListView_EditLabel (hwndListview , ItemID);

 return 0;
}

/****************************************************************************/
