/****************************************************************************/

#include "winlib.h"
#include <commctrl.h>
#include <shlobj.h>
#include "nfitypes.h"
#include "nfi.rh"

/****************************************************************************/

// These defines are used to specify statusbar panes
#define STATBAR_PANE1     0
#define STATBAR_PANE2     1
#define STATBAR_PANE3     2
#define STATBAR_PANE4     3
#define STATBAR_PANEALL 255

// These are used to tell the progress dialog box what caption it should give
// the dialog before it gets displayed
#define OPEN_PROGRESS     1
#define SAVE_PROGRESS     2
#define NEW_PROGRESS      3
#define DELETE_PROGRESS   4
#define MOVE_PROGRESS     5
#define EXTRACT_PROGRESS  6
#define ADD_PROGRESS      7
#define SORT_PROGRESS     8

/****************************************************************************/

// This holds the global setup information for the program. Everything in this
// struct also exsts in the registry and is loaded on startup, and saved when
// it changes.
typedef struct {
    UINT AddSpaceProcess;
    UINT ExtractSpaceProcess;
    BOOL LowercaseNames;
    BOOL SaveWindowPos;
    BOOL ReopenLast;
    BOOL RecurseDirs;
    BOOL MaintainDirStructure;
    char ExtractDir[MAX_PATH];
    char AddDir[MAX_PATH];
    char LastFileName[MAX_PATH];
} SetupStruct;

// This is the definition of a node in the path information tree. This is a
// binary tree which holds a list of all the unique paths that are in the
// currently open NFI file, as well as one for the root directory (/), one
// for whatever paths the user has tried to add to this session, and any paths
// left over from previously loaded NFI files.
typedef struct PathInfoTAG {
    char        *Path;
    PathInfoTAG *Left;
    PathInfoTAG *Right;
} PathInfo;

// This is the definition of a node in the path information tree. This is a
// singly linked list which is populated with a list of all the files which
// are supposed to be added to the NFI file. This is a doubly linked list and
// stores the file to add (AddFrom), the internal name and path of the file.
// Finally, there is an entry of type RealEntry. This is unused except when
// the file is saved. On the first pass through the save, the directory is
// saved. At that time, any file waiting to be added gets a real directory
// entry saved for it and stored in that member. On the second pass through
// the actual file data is written. Once that is done for an added entry,
// the listview node is set to use the RealEntry instead of the AddedFile
// entry. Finally all the AddedFile's are deleteed from memory and as far
// as the program knows, it has just finished loading a file.
typedef struct AddedFileTAG {
    char         *AddFrom;
    char         *InternalName;
    char         *InternalPath;
    AddedFileTAG *Next;
    AddedFileTAG *Prev;
} AddedFile;

/****************************************************************************/

// Global variable definitions
extern HINSTANCE   hInst;
extern HACCEL      hAccelTable;
extern char        szAppName[];
extern char        szTitle[];
extern HBITMAP     hToolbarBitmap;
extern HBITMAP     hOKBitmap;
extern HBITMAP     hCANCELBitmap;
extern HBITMAP     hADDFILEBitmap;
extern HBITMAP     hADDDIRBitmap;
extern HBITMAP     hREMOVEBitmap;
extern HBITMAP     hBROWSEBitmap;
extern HWND        hwndMain;
extern HWND        hwndToolbar;
extern HWND        hwndStatusbar;
extern HWND        hwndListview;
extern HWND        hwndProgressDlg;
extern HWND        hwndProgressCtrl;
extern HWND        hwndTextCtrl;
extern HWND        hwndAddListBox;
extern BOOL        FileIsDirty;
extern char        Filename[];
extern HANDLE      MainNFIFile;
extern LONG        FilesInNFI;
extern LONG        FilesToAdd;
extern LONG        FileSelectedCount;
extern NFIDirEntry *DirectoryRoot;
extern PathInfo    *NFIPathInfo;
extern AddedFile   *AddedFileList;
extern int         ExtractProcess;
extern SetupStruct Setup;

/****************************************************************************/

// Global function prototypes
BOOL     InitApplication         (HINSTANCE , char * , int);
BOOL     CenterWindow            (HWND , HWND);
void     MergeFilename           (HWND , char * , char *);
BOOL     CreateToolbar           (HWND);
BOOL     CreateStatusbar         (HWND);
void     InitializeStatusbar     (HWND);
void     UpdateStatusbar         (char * , WORD , WORD);
void     UpdateFileCount         (void);
void     UpdateSelectedCount     (void);
void     SetFileStatus           (BOOL);
BOOL     CreateListview          (HWND);
void     IdleProcessing          (void);
int      PromptToSave            (int TitleId , int PromptId);
void     PumpMessages            (void);
PathInfo *AddPathToTree          (PathInfo *Under , char *Path);
void     DeletePathTree          (void);
void     AddPathToComboBox       (PathInfo *Node , HWND Control);
void     UpdateAddedFileList     (void);
void     UnlinkDirEntry          (NFIDirEntry *Entry);
void     UnlinkAddEntry          (AddedFile *Entry);
void     CleanTemporaryDirectory (void);
void     GetTemporaryDirectory   (char *Buffer);
BOOL     ExtractEntry            (LV_ITEM &Item , BOOL ToTemp , char *Name);
void     GetHelpFileName         (char *Filename);

// File operation functions
void DoSave                (void);
void DoSaveAs              (void);
void DoOpen                (BOOL);
void DoActualOpenOperation (void);
void DoNew                 (void);
void DoExtraction          (void);

// Registry functions
void SaveBOOLToRegistry              (BOOL &Var , char *Key);
BOOL LoadBOOLFromRegistry            (BOOL &Var , char *Key);
void SaveUINTToRegistry              (UINT &Var , char *Key);
BOOL LoadUINTFromRegistry            (UINT &Var , char *Key);
void SaveWINDOWPLACEMENTToRegistry   (WINDOWPLACEMENT &Var , char *Key);
BOOL LoadWINDOWPLACEMENTFromRegistry (WINDOWPLACEMENT &Var , char *Key);
void SavePSZToRegistry               (char *Var , UINT Size , char *Key);
BOOL LoadPSZFromRegistry             (char *Var , UINT Size , char *Key);
void SaveSetupStructToRegistry       (SetupStruct &Var);
void LoadSetupStructFromRegistry     (SetupStruct &Var);
void SaveMainWindowPosition          (void);
BOOL LoadMainWindowPostion           (WINDOWPLACEMENT &wp);
void CreateAssociation               (void);

// Global window callback definitions
LRESULT CALLBACK WndProc      (HWND , UINT , WPARAM , LPARAM);
LRESULT CALLBACK AboutProc    (HWND , UINT , WPARAM , LPARAM);
LRESULT CALLBACK MoveProc     (HWND , UINT , WPARAM , LPARAM);
LRESULT CALLBACK ExtractProc  (HWND , UINT , WPARAM , LPARAM);
LRESULT CALLBACK AddProc      (HWND , UINT , WPARAM , LPARAM);
LRESULT CALLBACK OptionProc   (HWND , UINT , WPARAM , LPARAM);
LRESULT CALLBACK ProgressProc (HWND , UINT , WPARAM , LPARAM);

/****************************************************************************/
