/****************************************************************************/

#include "nfi.h"

/****************************************************************************/

// This is the message dispatching class for the option dialog box
class COptionDialogDispatcher : public CDispatcher {
public:
    COptionDialogDispatcher (handlerType Handler) : CDispatcher (Handler) {}

    LRESULT MsgInit (HWND, UINT, WPARAM, LPARAM);
    LRESULT MsgHelp (HWND, UINT, WPARAM, LPARAM);

    LRESULT CmdDone (HWND, WORD, WORD, HWND);

MESSAGE_LIST;
COMMAND_LIST;
};

START_MESSAGE_LIST (COptionDialogDispatcher)
    MESSAGE (WM_HELP       , MsgHelp)
    MESSAGE (WM_INITDIALOG , MsgInit)
END_MESSAGE_LIST

START_COMMAND_LIST (COptionDialogDispatcher)
    COMMAND (IDOK     , CmdDone)
    COMMAND (IDCANCEL , CmdDone)
END_COMMAND_LIST

/****************************************************************************/

static COptionDialogDispatcher DialogDispatch (handlerNone);

/*****************************************************************************
 * Name         - OptionProc
 * Purpose      - This handles messages for the option dialog box by using it's
 *                dispatcher class.
 * Arguments    - Standard window procedure dialog boxes
 * Return Value - Depends on the message
 ****************************************************************************/
LRESULT CALLBACK OptionProc (HWND hDlg , UINT uMessage ,
                             WPARAM wParam, LPARAM lParam)
{
 return DialogDispatch.dispatchMessage (hDlg , uMessage , wParam , lParam);
}

/*****************************************************************************
 * Name         - COptionDialogDispatcher::MsgInit
 * Purpose      - This handles the WM_INITDIALOG message.
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT COptionDialogDispatcher::MsgInit (HWND hDlg , UINT /*uMessage*/ ,
                                          WPARAM /*wParam*/ , LPARAM /*lParam*/)
{
 // Tell the button to use the bitmap we loaded.
 SendMessage (GetDlgItem (hDlg , IDOK) , BM_SETIMAGE , (WPARAM) IMAGE_BITMAP , (LPARAM) hOKBitmap);
 SendMessage (GetDlgItem (hDlg , IDCANCEL) , BM_SETIMAGE , (WPARAM) IMAGE_BITMAP , (LPARAM) hCANCELBitmap);

 // Check the proper "Space Import" parameter based on the global setup
 // options.
 CheckDlgButton (hDlg , IDC_KEEPSPACES + Setup.AddSpaceProcess , BST_CHECKED);

 SetFocus (GetDlgItem (hDlg , IDC_KEEPSPACES + Setup.AddSpaceProcess));

 // Check the checkbutton buttons depending on the options state
 CheckDlgButton (hDlg , IDC_MAKELOWER , Setup.LowercaseNames ? BST_CHECKED : BST_UNCHECKED);
 CheckDlgButton (hDlg , IDC_SAVEPOS , Setup.SaveWindowPos ? BST_CHECKED : BST_UNCHECKED);
 CheckDlgButton (hDlg , IDC_REOPENLAST , Setup.ReopenLast ? BST_CHECKED : BST_UNCHECKED);

 // Center the dialog over the application window
 CenterWindow (hDlg , GetWindow (hDlg , GW_OWNER));

 return TRUE;
}

/*****************************************************************************
 * Name         - COptionDialogDispatcher::MsgHelp
 * Purpose      - This handles the WM_HELP message.
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT COptionDialogDispatcher::MsgHelp (HWND hDlg , UINT /*uMessage*/ ,
                                          WPARAM /*wParam*/ , LPARAM lParam)
{
 static DWORD HelpID[] = {
     IDC_MAKELOWER     , IDC_MAKELOWER     ,
     IDC_SAVEPOS       , IDC_SAVEPOS       ,
     IDC_REOPENLAST    , IDC_REOPENLAST    ,
     IDC_KEEPSPACES    , IDC_KEEPSPACES    ,
     IDC_CONVERTSPACES , IDC_CONVERTSPACES ,
     IDC_REMOVESPACES  , IDC_REMOVESPACES  ,
     IDOK              , IDC_OPTIONOK      ,
     IDCANCEL          , IDC_OPTIONCANCEL  ,
     0                 , 0 ,
 };

 char HelpFile[MAX_PATH];
 GetHelpFileName (HelpFile);

 HELPINFO hi = *(LPHELPINFO) lParam;

 if (hi.iContextType == HELPINFO_WINDOW &&
     hDlg != hi.hItemHandle) {
         WinHelp ((HWND) hi.hItemHandle ,
                  HelpFile ,
                  HELP_WM_HELP ,
                  (DWORD) (LPVOID) HelpID);
 }

 return TRUE;
}

/*****************************************************************************
 * Name         - COptionDialogDispatcher::CmdDone
 * Purpose      - This function handles command messages for the button and
 *                the standard windows close controls.
 * Arguments    - hDlg: Handle of the dialog box
 *                wCommand: Command ID of the pressed button
 * Return Value - TRUE
 ****************************************************************************/
LRESULT COptionDialogDispatcher::CmdDone (HWND hDlg, WORD wCommand,
                                          WORD /*wNotify*/, HWND /*hwndCtrl*/)
{
 if (wCommand == IDOK) {
     if (IsDlgButtonChecked (hDlg , IDC_KEEPSPACES) == BST_CHECKED)
         Setup.AddSpaceProcess = 0;
     else
         if (IsDlgButtonChecked (hDlg , IDC_CONVERTSPACES) == BST_CHECKED)
             Setup.AddSpaceProcess = 1;
         else
             Setup.AddSpaceProcess = 2;

     Setup.LowercaseNames = (IsDlgButtonChecked (hDlg , IDC_MAKELOWER) == BST_CHECKED);
     Setup.SaveWindowPos = (IsDlgButtonChecked (hDlg , IDC_SAVEPOS) == BST_CHECKED);
     Setup.ReopenLast = (IsDlgButtonChecked (hDlg , IDC_REOPENLAST) == BST_CHECKED);

     SaveSetupStructToRegistry (Setup);
 }

 // End the dialog
 EndDialog (hDlg , wCommand);
 return TRUE;
}

/****************************************************************************/

