/****************************************************************************/

#include "nfi.h"

/****************************************************************************/

// This is the message dispatching class for the progress dialog box
class CProgressDialogDispatcher : public CDispatcher {
public:
    CProgressDialogDispatcher (handlerType Handler) : CDispatcher (Handler) {}

    LRESULT MsgInit     (HWND, UINT, WPARAM, LPARAM);
    LRESULT MsgDestroy   (HWND, UINT, WPARAM, LPARAM);

MESSAGE_LIST;
};

START_MESSAGE_LIST (CProgressDialogDispatcher)
    MESSAGE (WM_INITDIALOG , MsgInit)
    MESSAGE (WM_DESTROY    , MsgDestroy)
END_MESSAGE_LIST

/****************************************************************************/

static CProgressDialogDispatcher DialogDispatch (handlerNone);

/*****************************************************************************
 * Name         - ProgressProc
 * Purpose      - This handles messages for the progress dialog box by using
 *                it's dispatcher class.
 * Arguments    - Standard window procedure dialog boxes
 * Return Value - Depends on the message
 ****************************************************************************/
LRESULT CALLBACK ProgressProc (HWND hDlg , UINT uMessage ,
                               WPARAM wParam, LPARAM lParam)
{
 return DialogDispatch.dispatchMessage (hDlg , uMessage , wParam , lParam);
}

/*****************************************************************************
 * Name         - CProgressDialogDispatcher::MsgInit
 * Purpose      - This handles the WM_INITDIALOG message.
 * Arguments    - hDlg: Handle of the dialog box
 * Return Value - TRUE
 ****************************************************************************/
LRESULT CProgressDialogDispatcher::MsgInit (HWND hDlg , UINT /*uMessage*/ ,
                                         WPARAM /*wParam*/ , LPARAM lParam)
{
 WINASSERT (hwndProgressDlg == NULL);

 // Set the global variable to the handle of the dialog box
 hwndProgressDlg = hDlg;
 hwndProgressCtrl = GetDlgItem (hDlg , IDC_PROGRESSBAR);
 hwndTextCtrl = GetDlgItem (hDlg , IDC_PROGRESSTEXT);

 switch (lParam) {
     case OPEN_PROGRESS: {
         CResString Title (hInst , IDS_OPENPROGTITLE);
         SetWindowText (hDlg , Title);
         break;
     }

     case SAVE_PROGRESS: {
         CResString Title (hInst , IDS_SAVEPROGTITLE);
         SetWindowText (hDlg , Title);
         break;
     }

     case NEW_PROGRESS: {
         CResString Title (hInst , IDS_NEWPROGTITLE);
         SetWindowText (hDlg , Title);
         break;
     }

     case DELETE_PROGRESS: {
         CResString Title (hInst , IDS_DELETEPROGTITLE);
         SetWindowText (hDlg , Title);
         break;
     }

     case MOVE_PROGRESS: {
         CResString Title (hInst , IDS_MOVEPROGTITLE);
         SetWindowText (hDlg , Title);
         break;
     }

     case EXTRACT_PROGRESS: {
         CResString Title (hInst , IDS_EXTRACTPROGTITLE);
         SetWindowText (hDlg , Title);
         break;
     }

     case SORT_PROGRESS: {
         CResString Title (hInst , IDS_SORTPROGTITLE);
         SetWindowText (hDlg , Title);
         break;
     }

     case ADD_PROGRESS: {
         CResString Title (hInst , IDS_ADDPROGTITLE);
         SetWindowText (hDlg , Title);
         ShowWindow (hwndProgressCtrl , SW_HIDE);
         break;
     }
 }

 // Center the dialog over the application window
 CenterWindow (hDlg , GetWindow (hDlg , GW_OWNER));

 // Disable the listview so nobody can fiddle with it
 EnableWindow (hwndListview , FALSE);

 return TRUE;
}

/*****************************************************************************
 * Name         - CProgressDialogDispatcher::MsgDestroy
 * Purpose      - Handles destroy messages for the dialog box
 * Arguments    - Message arguments for WM_DESTROY
 * Return Value - 0
 ****************************************************************************/
LRESULT CProgressDialogDispatcher::MsgDestroy (HWND /*hDlg*/ , UINT /*uMessage*/ ,
                                               WPARAM /*wParam*/ , LPARAM /*lParam*/)
{
 // Set the global progress variables to NULL since they are not valid anymore.
 hwndProgressDlg = NULL;
 hwndProgressCtrl = NULL;
 hwndTextCtrl = NULL;

 // Reenable the window
 EnableWindow (hwndListview , TRUE);

 return 0;
}

/****************************************************************************/

