/****************************************************************************/

#include "nfi.h"

/****************************************************************************/

#define KEY_MAINREG       "software\\Black Spider\\NFI Tool 1.0"
#define KEY_ADDSPACE      "ImportSpaceProcessing"
#define KEY_EXTRACTSPACE  "ExportSpaceProcessing"
#define KEY_LOWERNAMES    "ImportAsLowercase"
#define KEY_SAVEPOS       "SaveMainWindowPosition"
#define KEY_SAVEDPOSITION "MainWindowPos"
#define KEY_OPENLAST      "OpenLastFileOnRestart"
#define KEY_RECURSE       "RecurseDirectoriesOnImport"
#define KEY_MAINTAINDIR   "MaintainDirectoriesOnExport"
#define KEY_EXTRACTDIR    "ExtractionDefaultDir"
#define KEY_ADDDIR        "ImportDefaultNFIDir"
#define KEY_LASTFILE      "LastOpenFilename"

#define KEY_ASSOCINFO     "NFIBinaryArchivedFile"
#define KEY_DEFAULTICON   "DefaultIcon"
#define KEY_COMMAND       "shell\\open\\command"

/*****************************************************************************
 * Name         - SaveBOOLToRegistry
 * Purpose      - Saves a boolean value to the registry using the given key.
 * Arguments    - Var: Boolean var to save
 *                Key: Key to save the given var in
 * Return Value - None
 ****************************************************************************/
void SaveBOOLToRegistry (BOOL &Var , char *Key)
{
 WINASSERT (Key);
 HKEY RegKey;

 // Try to create the key that we will be saving our information into. If the
 // key doesn't exist it will then be created. If it does exist it will be
 // opened instead.
 if (RegCreateKeyEx (HKEY_CURRENT_USER , KEY_MAINREG , 0 , szAppName , 0 ,
                     KEY_ALL_ACCESS , NULL , &RegKey ,
                     NULL) == ERROR_SUCCESS) {

     // If we managed to open or create our key, we set the user mode key.
     // This will either create the key if it doesn't exist or update it if it
     // does. Once we are done we close the key.
     RegSetValueEx (RegKey , Key , 0 , REG_BINARY , (LPBYTE) &Var ,
                    sizeof (BOOL));
     RegCloseKey (RegKey);
 }
}

/*****************************************************************************
 * Name         - LoadBOOLFromRegistry
 * Purpose      - Loads a boolean value from the registry
 * Arguments    - Var: Boolean var to put the read value into.
 *                Key: Key to load the given var from
 * Return Value - TRUE if the bool was read successfully
 *                FALSE if the bool could not be read. The var parameter is
 *                unchanged.
 ****************************************************************************/
BOOL LoadBOOLFromRegistry (BOOL &Var , char *Key)
{
 WINASSERT (Key);
 HKEY  RegKey   = 0;
 BOOL  Worked   = FALSE;
 BOOL  ReadVal;
 DWORD DataSize = sizeof (BOOL);

 // Try to open up the registry key.
 if (RegOpenKeyEx (HKEY_CURRENT_USER , KEY_MAINREG , 0 , KEY_READ ,
                   &RegKey) == ERROR_SUCCESS) {
     // Try to get the value. We know it worked if the function returned ok
     // AND the size of the data we received was proper.
     if (RegQueryValueEx (RegKey , Key , 0 , NULL ,
                          (LPBYTE) &ReadVal , &DataSize) == ERROR_SUCCESS)
         Worked = (DataSize == sizeof (BOOL));

     // If the function worked set the parameter we will be using to return
     // the read value back to the caller.
     if (Worked)
         Var = ReadVal;

     // Close the registry key.
     RegCloseKey (RegKey);
 }

 // Return a value indicating if it worked or not.
 return Worked;
}

/*****************************************************************************
 * Name         - SaveUINToRegistry
 * Purpose      - Saves a UINT value to the registry using the given key.
 * Arguments    - Var: UINT var to save
 *                Key: Key to save the given var in
 * Return Value - None
 ****************************************************************************/
void SaveUINTToRegistry (UINT &Var , char *Key)
{
 WINASSERT (Key);
 HKEY RegKey;

 // Try to create the key that we will be saving our information into. If the
 // key doesn't exist it will then be created. If it does exist it will be
 // opened instead.
 if (RegCreateKeyEx (HKEY_CURRENT_USER , KEY_MAINREG , 0 , szAppName , 0 ,
                     KEY_ALL_ACCESS , NULL , &RegKey ,
                     NULL) == ERROR_SUCCESS) {

     // If we managed to open or create our key, we set the user mode key.
     // This will either create the key if it doesn't exist or update it if it
     // does. Once we are done we close the key.
     RegSetValueEx (RegKey , Key , 0 , REG_BINARY , (LPBYTE) &Var ,
                    sizeof (UINT));
     RegCloseKey (RegKey);
 }
}

/*****************************************************************************
 * Name         - LoadUINTFromRegistry
 * Purpose      - Loads a UINT value from the registry
 * Arguments    - Var: UINT var to put the read value into.
 *                Key: Key to load the given var from
 * Return Value - TRUE if the UINT was read successfully
 *                FALSE if the UINT could not be read. The var parameter is
 *                unchanged.
 ****************************************************************************/
BOOL LoadUINTFromRegistry (UINT &Var , char *Key)
{
 WINASSERT (Key);
 HKEY  RegKey   = 0;
 BOOL  Worked   = FALSE;
 UINT  ReadVal;
 DWORD DataSize = sizeof (BOOL);

 // Try to open up the registry key.
 if (RegOpenKeyEx (HKEY_CURRENT_USER , KEY_MAINREG , 0 , KEY_READ ,
                   &RegKey) == ERROR_SUCCESS) {
     // Try to get the value. We know it worked if the function returned ok
     // AND the size of the data we received was proper.
     if (RegQueryValueEx (RegKey , Key , 0 , NULL ,
                          (LPBYTE) &ReadVal , &DataSize) == ERROR_SUCCESS)
         Worked = (DataSize == sizeof (UINT));

     // If the function worked set the parameter we will be using to return
     // the read value back to the caller.
     if (Worked)
         Var = ReadVal;

     // Close the registry key.
     RegCloseKey (RegKey);
 }

 // Return a value indicating if it worked or not.
 return Worked;
}

/*****************************************************************************
 * Name         - SaveWINDOWPLACEMENTToRegistry
 * Purpose      - Saves a WINDOWPLACEMENT struct to the registry using the
 *                given key.
 * Arguments    - Var: Struct to save
 *                Key: Key to save the given var in
 * Return Value - None
 ****************************************************************************/
void SaveWINDOWPLACEMENTToRegistry (WINDOWPLACEMENT &Var , char *Key)
{
 WINASSERT (Key);
 HKEY RegKey;

 // Try to create the key that we will be saving our information into. If the
 // key doesn't exist it will then be created. If it does exist it will be
 // opened instead.
 if (RegCreateKeyEx (HKEY_CURRENT_USER , KEY_MAINREG , 0 , szAppName , 0 ,
                     KEY_ALL_ACCESS , NULL , &RegKey ,
                     NULL) == ERROR_SUCCESS) {

     // If we managed to open or create our key, we set the user mode key.
     // This will either create the key if it doesn't exist or update it if it
     // does. Once we are done we close the key.
     RegSetValueEx (RegKey , Key , 0 , REG_BINARY , (LPBYTE) &Var ,
                    sizeof (WINDOWPLACEMENT));
     RegCloseKey (RegKey);
 }
}

/*****************************************************************************
 * Name         - LoadWINDOWPLACEMENTFromRegistry
 * Purpose      - Loads a WINDOWPLACEMENT struct from the registry
 * Arguments    - Var: WINDOWPLACEMENT var to put the read value into.
 *                Key: Key to load the given var from
 * Return Value - TRUE if the WINDOWPLACEMENT  was read successfully
 *                FALSE if the WINDOWPLACEMENT could not be read. The var
 *                parameter is unchanged.
 ****************************************************************************/
BOOL LoadWINDOWPLACEMENTFromRegistry (WINDOWPLACEMENT &Var , char *Key)
{
 WINASSERT (Key);
 HKEY  RegKey   = 0;
 BOOL  Worked   = FALSE;
 WINDOWPLACEMENT ReadVal;
 DWORD DataSize = sizeof (WINDOWPLACEMENT);

 // Try to open up the registry key.
 if (RegOpenKeyEx (HKEY_CURRENT_USER , KEY_MAINREG , 0 , KEY_READ ,
                   &RegKey) == ERROR_SUCCESS) {
     // Once again, query the registry, and make sure that the function
     // returned ok and we got the right amount of data.
     if (RegQueryValueEx (RegKey , Key , 0 , NULL ,
                          (LPBYTE) &ReadVal , &DataSize) == ERROR_SUCCESS)
         Worked = (DataSize == sizeof (WINDOWPLACEMENT));

     // If it worked, set the return value to go back to the caller.
     if (Worked)
         Var = ReadVal;

     // Close the registry key
     RegCloseKey (RegKey);
 }

 // Return a value indicating if it worked or not.
 return Worked;
}

/*****************************************************************************
 * Name         - SavePSZToRegistry
 * Purpose      - Saves a string to the registry using the given key.
 * Arguments    - Var: string to save
 *                Size: Length of the string (TOTAL, not current)
 *                Key: Key to save the given var in
 * Return Value - None
 ****************************************************************************/
void SavePSZToRegistry (char *Var , UINT Size , char *Key)
{
 WINASSERT (Key);
 HKEY RegKey;

 // Try to create the key that we will be saving our information into. If the
 // key doesn't exist it will then be created. If it does exist it will be
 // opened instead.
 if (RegCreateKeyEx (HKEY_CURRENT_USER , KEY_MAINREG , 0 , szAppName , 0 ,
                     KEY_ALL_ACCESS , NULL , &RegKey ,
                     NULL) == ERROR_SUCCESS) {

     // If we managed to open or create our key, we set the user mode key.
     // This will either create the key if it doesn't exist or update it if it
     // does. Once we are done we close the key.
     RegSetValueEx (RegKey , Key , 0 , REG_SZ , (LPBYTE) Var , Size);
     RegCloseKey (RegKey);
 }
}

/*****************************************************************************
 * Name         - LoadPSZFromRegistry
 * Purpose      - Loads a string from the registry
 * Arguments    - Var: string var to put the read value into.
 *                Size: Size of the var parameter, including space for the
 *                      null terminator
 *                Key: Key to load the given var from
 * Return Value - TRUE if the WINDOWPLACEMENT  was read successfully
 *                FALSE if the WINDOWPLACEMENT could not be read. The var
 *                parameter is unchanged.
 ****************************************************************************/
BOOL LoadPSZFromRegistry (char *Var , UINT Size , char *Key)
{
 WINASSERT (Key);
 HKEY  RegKey   = 0;
 DWORD DataSize = Size;
 BOOL  Worked=FALSE;

 // Try to open up the registry key.
 if (RegOpenKeyEx (HKEY_CURRENT_USER , KEY_MAINREG , 0 , KEY_READ ,
                   &RegKey) == ERROR_SUCCESS) {
     // Once again, query the registry, and make sure that the function
     // returned ok and we got the right amount of data.
     if (RegQueryValueEx (RegKey , Key , 0 , NULL ,
                          (LPBYTE) Var , &DataSize) == ERROR_SUCCESS)
         Worked = TRUE;

     // Close the registry key
     RegCloseKey (RegKey);
 }

 // Return a value indicating if it worked or not.
 return Worked;
}

/*****************************************************************************
 * Name         - SaveMainWindowPosition
 * Purpose      - This saves the position of the main window into the registry
 * Arguments    - None
 * Return Value - None
 ****************************************************************************/
void SaveMainWindowPosition (void)
{
 WINDOWPLACEMENT wp;

 wp.length = sizeof (WINDOWPLACEMENT);
 GetWindowPlacement (hwndMain , &wp);
 SaveWINDOWPLACEMENTToRegistry (wp , KEY_SAVEDPOSITION);
}

/*****************************************************************************
 * Name         - LoadMainWindowPosition
 * Purpose      - This gets the main window position out of the registry and
 *                puts it into the passed parameter. If the information
 *                cannot be loaded, the passed in variable is not altered.
 * Arguments    - wp: Handle to the WINDOWPLACEMENT structure to fill up with
 *                    the window position information
 * Return Value - TRUE on success or FALSE on failure
 ****************************************************************************/
BOOL LoadMainWindowPostion (WINDOWPLACEMENT &wp)
{
 return LoadWINDOWPLACEMENTFromRegistry (wp , KEY_SAVEDPOSITION);
}

/*****************************************************************************
 * Name         - SaveSetupStructToRegistry
 * Purpose      - Saves a setup structure to the registry
 * Arguments    - Var: Setup struct to save
 * Return Value - None
 ****************************************************************************/
void SaveSetupStructToRegistry (SetupStruct &Var)
{
 SaveUINTToRegistry (Var.AddSpaceProcess , KEY_ADDSPACE);
 SaveUINTToRegistry (Var.ExtractSpaceProcess , KEY_EXTRACTSPACE);
 SaveBOOLToRegistry (Var.LowercaseNames , KEY_LOWERNAMES);
 SaveBOOLToRegistry (Var.SaveWindowPos , KEY_SAVEPOS);
 SaveBOOLToRegistry (Var.ReopenLast , KEY_OPENLAST);
 SaveBOOLToRegistry (Var.RecurseDirs , KEY_RECURSE);
 SaveBOOLToRegistry (Var.MaintainDirStructure , KEY_MAINTAINDIR);
 SavePSZToRegistry  (Var.ExtractDir , sizeof (Var.ExtractDir) , KEY_EXTRACTDIR);
 SavePSZToRegistry  (Var.AddDir , sizeof (Var.AddDir) , KEY_ADDDIR);
 SavePSZToRegistry  (Var.LastFileName , sizeof (Var.LastFileName) , KEY_LASTFILE);
}

/*****************************************************************************
 * Name         - LoadSetupStructToRegistry
 * Purpose      - Loads a setup structure from the registry
 * Arguments    - Var: Setup struct to put the setup information into
 * Return Value - None
 ****************************************************************************/
void LoadSetupStructFromRegistry (SetupStruct &Var)
{
 LoadUINTFromRegistry (Var.AddSpaceProcess , KEY_ADDSPACE);
 LoadUINTFromRegistry (Var.ExtractSpaceProcess , KEY_EXTRACTSPACE);
 LoadBOOLFromRegistry (Var.LowercaseNames , KEY_LOWERNAMES);
 LoadBOOLFromRegistry (Var.SaveWindowPos , KEY_SAVEPOS);
 LoadBOOLFromRegistry (Var.ReopenLast , KEY_OPENLAST);
 LoadBOOLFromRegistry (Var.RecurseDirs , KEY_RECURSE);
 LoadBOOLFromRegistry (Var.MaintainDirStructure , KEY_MAINTAINDIR);
 LoadPSZFromRegistry  (Var.ExtractDir , sizeof (Var.ExtractDir) , KEY_EXTRACTDIR);
 LoadPSZFromRegistry  (Var.AddDir , sizeof (Var.AddDir) , KEY_ADDDIR);
 LoadPSZFromRegistry  (Var.LastFileName , sizeof (Var.LastFileName) , KEY_LASTFILE);
}

/*****************************************************************************
 * Name         - MakeIconEntry
 * Purpose      - This creates a DefaultIcon key under the given parent key
 *                and sets it to use the first icon present in the given
 *                module name, which should be set to the complete path name
 *                to the application.
 * Arguments    - ParentKey: Parent key to save the icon information under.
 *                ModuleName: String containing the complete path to the
 *                            application.
 * Return Value - None
 ****************************************************************************/
void MakeIconEntry (HKEY ParentKey , char *ModuleName)
{
 WINASSERT (ModuleName);

 char KeyValue[MAX_PATH];
 strcpy (KeyValue , ModuleName);
 strcat (KeyValue , ",0");

 HKEY RegKey;

 // Try to create the key that we will be saving our information into.
 if (RegCreateKeyEx (ParentKey , KEY_DEFAULTICON , 0 , szAppName , 0 ,
                     KEY_ALL_ACCESS , NULL , &RegKey ,
                     NULL) == ERROR_SUCCESS) {

     // If it worked, set the DefaultIcon key.
     RegSetValueEx (RegKey , "" , 0 , REG_SZ , (LPBYTE) KeyValue , sizeof (KeyValue));
     RegCloseKey (RegKey);
 }
}

/*****************************************************************************
 * Name         - MakeCommandEntry
 * Purpose      - This creates a shell\open\command key under the given key
 *                and sets it's value to "ModuleName %1". This allows the user
 *                to double click a file of our time and have the program
 *                open it.
 * Arguments    - ParentKey: The key to save the command information under.
 *                ModuleName: String containing the complete path to the
 *                            application.
 * Return Value - None
 ****************************************************************************/
static void MakeCommandEntry (HKEY ParentKey , char *ModuleName)
{
 char KeyValue[MAX_PATH];
 strcpy (KeyValue , ModuleName);
 strcat (KeyValue , " %1");

 HKEY RegKey;

 // Try to create the key we will use
 if (RegCreateKeyEx (ParentKey , KEY_COMMAND , 0 , szAppName , 0 ,
                     KEY_ALL_ACCESS , NULL , &RegKey ,
                     NULL) == ERROR_SUCCESS) {

     // If it worked, save the command info to this key
     RegSetValueEx (RegKey , "" , 0 , REG_SZ , (LPBYTE) KeyValue , sizeof (KeyValue));
     RegCloseKey (RegKey);
 }
}

/*****************************************************************************
 * Name         - MakeExtensionAssociation
 * Purpose      - This function performs the last step in the association
 *                creation. It creates an entry for the extension of our file
 *                type and sets it's value to point to the registry key that
 *                contains all of the information about the assocation.
 * Arguments    - None
 * Return Value - None
 ****************************************************************************/
static void MakeExtensionAssociation (void)
{
 CResString DefExt (hInst , IDS_DEFAULTEXTENSION);
 char *KeyName = new char [strlen (DefExt) + 2];
 strcpy (KeyName , ".");
 strcat (KeyName , DefExt);

 HKEY RegKey;

 // Try to create the key that we will be saving our information into.
 if (RegCreateKeyEx (HKEY_CLASSES_ROOT , KeyName , 0 , szAppName , 0 ,
                     KEY_ALL_ACCESS , NULL , &RegKey ,
                     NULL) == ERROR_SUCCESS) {

     // If it worked, save the associated key value that goes with this
     // extension.
     RegSetValueEx (RegKey , "" , 0 , REG_SZ , (LPBYTE) KEY_ASSOCINFO , sizeof (KEY_ASSOCINFO));
     RegCloseKey (RegKey);
 }


 delete[] KeyName;
}

/*****************************************************************************
 * Name         - CreateAssociation
 * Purpose      - This function creates an assocation for the NFI file type
 *                in the registry that points to the module of the current
 *                instance of the program. The assocation is done in two
 *                parts.
 *
 *                First thing is we pick some key that is under
 *                HKEY_CLASSES_ROOT. That key has subkeys under it which
 *                identify the icon to use for files of this type and the
 *                commands that can be executed for it by the shell. It also
 *                contains the description of the file type, as seen in a
 *                detailed shell view window.
 *
 *                Second thing is to create a key, again under
 *                HKEY_CLASSES_ROOT, that is named .EXT, where EXT is the
 *                extension to associate, and which has a value equal to the
 *                first key we created. This allows the shell to point from
 *                the file extension, which it has, to the key entry that has
 *                the info on it.
 * Arguments    - None
 * Return Value - None
 ****************************************************************************/
void CreateAssociation (void)
{
 HKEY RegKey;

 char ExeName[MAX_PATH];
 char Description[MAX_PATH];
 CResString Desc (hInst , IDS_FILEDESCRIPTION , Description , sizeof (Description));


 // Get the module's EXE name. Return if it doesn't work
 if (!GetModuleFileName (hInst , ExeName , sizeof (ExeName)))
     return;

 // Try to create the key that we will be saving the actual association
 // information into.
 if (RegCreateKeyEx (HKEY_CLASSES_ROOT , KEY_ASSOCINFO , 0 , szAppName , 0 ,
                     KEY_ALL_ACCESS , NULL , &RegKey ,
                     NULL) == ERROR_SUCCESS) {

     // If it worked, save the description of our NFI file as the value
     // of this key.
     RegSetValueEx (RegKey , "" , 0 , REG_SZ , (LPBYTE) Description , strlen (Description));

     // Now, tell it what icon to use for files of this type, which is, by
     // default the first icon in the EXE file. Which is the main icon,
     // so Yay.
     MakeIconEntry (RegKey , ExeName);

     // Now make the shell open command entry for this.
     MakeCommandEntry (RegKey , ExeName);

     // Close up the key, we're done with it now.
     RegCloseKey (RegKey);
 }

 // Now that we have the association info ready, make the actual assocation.
 MakeExtensionAssociation ();
}

/****************************************************************************/
