/****************************************************************************/

#include "nfi.h"

#define TEMP_ADDITIONAL_PATH "NFITool\\"

/*****************************************************************************
 * Name         - GetTemporaryDirectory
 * Purpose      - Gets the temporary directory that we use to store our files.
 *                This is the system temp directory with additional subdir
 *                information attached onto the end of it. The first time that
 *                the function is called in the program it creates the directory
 *                itself. (It assumes the temporary directory exists). The
 *                Code assumes that buffer is MAX_PATH characters in length.
 * Arguments    - Buffer: Char to put the info into
 * Return Value - None
 ****************************************************************************/
void GetTemporaryDirectory (char *Buffer)
{
 WINASSERT (Buffer);

 static BOOL Initialized = FALSE;

 GetTempPath (MAX_PATH , Buffer);

 // Make sure it has a trailing slash
 if (Buffer[strlen (Buffer) - 1] != '\\')
     strcat (Buffer , "\\");

 strcat (Buffer , TEMP_ADDITIONAL_PATH);

 // If we haven't been called before, make the directory now.
 if (!Initialized) {
     Initialized = TRUE;
     CreateDirectory (Buffer , NULL);
 }
}

/*****************************************************************************
 * Name         - CleanTemporaryDirectory
 * Purpose      - When this is called, it scans for every file which is
 *                contained inside the temporary directory and attempts to
 *                delete it. Once that is done it attempts to remove the
 *                directory itself. This may not work if a file cannot be
 *                deleted because it is still in use, but it will get
 *                deleted eventually.
 * Arguments    - None
 * Return Value - None
 ****************************************************************************/
void CleanTemporaryDirectory (void)
{
 char TempDir[MAX_PATH];
 char SearchPath[MAX_PATH];

 // Get the temporary directory that we store our files in.
 GetTemporaryDirectory (TempDir);

 // Make a search path
 strcpy (SearchPath , TempDir);
 strcat (SearchPath , "*.*");

 HANDLE          SearchHandle;
 WIN32_FIND_DATA wfd;
 BOOL            Done=FALSE;

 // Open a search handle for the temporary directory. If it does not work we
 // can only assume that the temporary directory does not exist at all, because
 // if it did there would at least be . and .. entries inside it. So we return
 // if the operation fails.
 SearchHandle = FindFirstFile (SearchPath , &wfd);
 if (SearchHandle == INVALID_HANDLE_VALUE)
     return;

 // Otherwise, keep looping until we are done.
 while (!Done) {
     // Is this a file?
     if ((wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0) {
         // Yes, try to delete it.
         char FName[MAX_PATH];
         strcpy (FName , TempDir);
         strcat (FName , wfd.cFileName);

         DeleteFile (FName);
     }

     // Find the next file.
     Done = !FindNextFile (SearchHandle , &wfd);
 }

 // Close the find handle.
 FindClose (SearchHandle);

 // Get rid of the temporary directory
 RemoveDirectory (TempDir);
}

/****************************************************************************/
