/****************************************************************************/

#include "nfi.h"

/****************************************************************************/

// These defines control the bitmap used to construct the toolbar.
// Specifically, how many images in it, how much of each image should be used
// per button and the size of a button. The button size argument is totally
// 300% useless because if you make it big without making the bitmap itself
// big it displays wrong. So, lesson learned. Always make the Button size
// default and use a bigger bitmap if you want larger buttons.
#define NUMIMAGES         8
#define IMAGEWIDTH       20
#define IMAGEHEIGHT      20
#define BUTTONWIDTH       0
#define BUTTONHEIGHT      0

// This is used in our global array of button configuration info to specify
// buttons that are disabled. There isn't one predefined for some reason.
#define TBSTATE_DISABLED  0

/****************************************************************************/

// This is the struct we used to make our toolbar buttons. There is one array
// entry per button, speficying (in order):
//
// 1) The 0 based offset into the bitmap to use for this button
// 2) The command ID of the button. Must be 0 if the button is a seperator
// 3) Flags for the button that specify it's state.
// 4) The sort of button this is. Can be check, radio, etc.
// 5) Application defined data to store in the button. We don't use this
// 6) ID of the string to display on the button. I don't know how that's used
//    and I also don't care. We don't use that either.
static TBBUTTON tbButton[] = {
    {0,  0,                  TBSTATE_ENABLED,  TBSTYLE_SEP,    0, 0},
    {0,  IDM_FILENEW,        TBSTATE_ENABLED,  TBSTYLE_BUTTON, 0, 0},
    {0,  0,                  TBSTATE_ENABLED,  TBSTYLE_SEP,    0, 0},
    {1,  IDM_FILEOPEN,       TBSTATE_ENABLED,  TBSTYLE_BUTTON, 0, 0},
    {2,  IDM_FILESAVE,       TBSTATE_DISABLED, TBSTYLE_BUTTON, 0, 0},
    {0,  0,                  TBSTATE_ENABLED,  TBSTYLE_SEP,    0, 0},
    {3,  IDM_ADDFILES,       TBSTATE_ENABLED,  TBSTYLE_BUTTON, 0, 0},
    {4,  IDM_DELETEFILES,    TBSTATE_DISABLED, TBSTYLE_BUTTON, 0, 0},
    {6,  IDM_VIEWFILES,      TBSTATE_DISABLED, TBSTYLE_BUTTON, 0, 0},
    {5,  IDM_EXTRACTFILES,   TBSTATE_DISABLED, TBSTYLE_BUTTON, 0, 0},
    {0,  0,                  TBSTATE_ENABLED,  TBSTYLE_SEP,    0, 0},
    {7,  IDM_MOVEFILES,      TBSTATE_DISABLED, TBSTYLE_BUTTON, 0, 0},
};

/*****************************************************************************
 * Name         - CreateToolbar
 * Purpose      - Creates the toolbar window
 * Arguments    - hWnd: handle to the parent window to use when making the
 *                      toolbar
 * Return Value - TRUE if the toolbar was made
 *                FALSE on error.
 ****************************************************************************/
BOOL CreateToolbar (HWND hWnd)
{
 WINASSERT (hWnd);
 WINASSERT (hwndToolbar == NULL);

 // Create the toolbar.
 hwndToolbar = CreateToolbarEx (hWnd,
                                WS_CHILD | WS_VISIBLE | TBSTYLE_TOOLTIPS ,
                                ID_TOOLBAR ,
                                NUMIMAGES ,
                                0 ,
                                (UINT) hToolbarBitmap ,
                                tbButton ,
                                sizeof (tbButton) / sizeof(TBBUTTON) ,
                                BUTTONWIDTH , BUTTONHEIGHT ,
                                IMAGEWIDTH , IMAGEHEIGHT ,
                                sizeof (TBBUTTON));

 WINASSERT (hwndToolbar);

 return (hwndToolbar != NULL);
}

/****************************************************************************/

